/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 54220 $
 * \date $Date: 2009-05-02 06:33:10 +0400 (Сбт, 02 Май 2009) $
 * \author $Author: lse $
 *
 * \brief       .
 */

#ifndef __CHAIN_UTIL_H
#define __CHAIN_UTIL_H

#include <set>
#include <string>

#include "asn1type.h"
#include "StoreUtil.h"
#include "ASN1Util.h"

class CertChainBuildFlags
{
public:
    //   CRL      
    static const unsigned REVOCATION_CHECK_END_CERT;
    //   CRL   ,  ,
    //    CRL
    static const unsigned REVOCATION_CHECK_CHAIN;
    //   CRL   Root  ,  
    //    CRL. Root    
    // CRL    .
    static const unsigned REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT;
    static const unsigned REVOCATION_CHECK_MASK;
    //        
    //      
    static const unsigned DISABLE_PASS1_QUALITY_FILTERING;
    //   CRL  ,    
    static const unsigned REVOCATION_CHECK_CACHE_ONLY;
    //  timeout   CRL
    static const unsigned REVOCATION_ACCUMULATIVE_TIMEOUT;
    //      
    //    
    static const unsigned CACHE_ONLY_URL_RETRIEVAL;
    unsigned flags;
};

class TrustStatus
{
public:
    // , ,  
    explicit TrustStatus() : value( 0 ) {}
    TrustStatus( const TrustStatus& src ) : value( src.value ) {}
    TrustStatus( unsigned src ) : value( src ) {}

    //   
    unsigned get_unsigned() const { return value; }

    //  CryptoAPI- 
    DWORD toCERT_TRUST_STATUS ();
    DWORD toRevocationResult();

    // 
    int compare( const TrustStatus &right ) const;

    //   
    std::string toString() const;
    std::wstring toWString() const;
    static TrustStatus fromString( const char *str );

    // 
    static const unsigned CT_NO_ERROR;
    static const unsigned IS_NOT_TIME_VALID;
    static const unsigned IS_REVOKED;
    static const unsigned IS_NOT_SIGNATURE_VALID;
    static const unsigned IS_NOT_VALID_FOR_USAGE;
    static const unsigned IS_UNTRUSTED_ROOT;
    static const unsigned INVALID_BASIC_CONSTRAINTS;
    static const unsigned IS_PARTIAL_CHAIN;
    static const unsigned REVOCATION_STATUS_UNKNOWN;

    //    
    static const unsigned VALUES[];
    static const char *STRINGS[];
#ifdef UNIX
    static const char *WSTRINGS[];
#else /* UNIX */
    static const wchar_t *WSTRINGS[];
#endif /* UNIX */
    static const size_t VALUE_LENGTH;

protected:
    int compare( const TrustStatus &src, unsigned flag ) const;
    unsigned value;
};

//   
inline bool operator== ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) == 0; }
inline bool operator!= ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) != 0; }
inline bool operator< ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) < 0; }
inline bool operator> ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) > 0; }
inline bool operator<= ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) <= 0; }
inline bool operator>= ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) >= 0; }

//   
class CertChainBuilder;

//   Store
typedef std::set<CertificateStore> CertificateStore_set;
//  Policy
typedef std::list<ASN1OBJID> external_policy_list;

extern const ASN1OBJID ANY_POLICY_OID;

//     
//  :
// -  : 
// - CRL store: CA, Root
// - Store: CA
// - Root store: Root
// -  . crl , Trust: 
// - Policy: POLICY_ANY
// -  : REVOCATION_CHECK_CHAIN
class CertChainContext
{
public:
    // , , 
    CertChainContext( bool add_root = true, bool extra_crl_stores = true,
	bool extra_cert_stores = true );
    ~CertChainContext();

    //       0
    void disable_any_policy();

    //  
    //    UTC time
    void set_date( const char *date );
    void set_usage( PCERT_USAGE_MATCH RequestedUsage );
    void set_date( const FILETIME& date );
    void set_list( const encoded_certificate_list *lst )
    { clear_list(); add_list( lst ); }
    void set_list( const ASN1TSeqOfList &extra )
    { clear_list(); add_list( extra ); }
    void set_trust( const encoded_certificate_list *lst )
    { clear_trust_list(); add_trust_list( lst ); }
    void set_crl_list( const encoded_crl_list *lst )
    { clear_crl_list(); add_crl_list( lst ); }
    void set_stores( const CertificateStore_set *stores)
    { clear_stores(); add_stores( stores ); }
    void set_crl_stores( const CertificateStore_set *stores)
    { clear_crl_stores(); add_crl_stores( stores ); }
    void set_initial_policies( const external_policy_list *policies )
    { clear_initial_policies(); add_initial_policies( policies ); }
    void set_initial_policy_mapping_inhibit( bool to );
    void set_initial_explicit_policy( bool to );
    void set_initial_any_policy_inhibit( bool to );
    void set_flags( unsigned f );
    void set_timeout( unsigned long time_value);

    //    
    void add_list( const CACMPT_BLOB *cert );
    void add_list( const encoded_certificate_list *lst );
    void add_list( const ASN1TSeqOfList &extra );
    void add_trust_list( const CACMPT_BLOB *cert );
    void add_trust_list( const encoded_certificate_list *lst );
    void add_crl_list( const CACMPT_BLOB *cert );
    void add_crl_list( const encoded_crl_list *lst );
    void add_stores( const wchar_t *store, bool issystem = false );
    void add_stores( const CertificateStore &store);
    void add_stores( const CertificateStore_set *stores);
    void add_stores( const store_handle& handle );
    void add_crl_stores( const wchar_t *store, bool issystem = false );
    void add_crl_stores( const CertificateStore &store);
    void add_crl_stores( const CertificateStore_set *stores);
    void add_crl_stores( const store_handle& handle );
    void add_initial_policies( const ASN1OBJID *policy );
    void add_initial_policies( const external_policy_list *policies );

    // old style
    void add_stores( const CertificateStore *stores, 
	size_t stores_quant );
    void add_crl_stores( const CertificateStore *stores, 
	size_t stores_quant );

    //    
    void clear_list();
    void clear_trust_list();
    void clear_crl_list();
    void clear_stores();
    void clear_crl_stores();
    void clear_initial_policies();

    //   
    void get_list( encoded_certificate_list *list ) const;
    void get_trust_list( encoded_certificate_list *list ) const;
    void get_crl_list( encoded_crl_list *list ) const;
    void get_initial_policies( external_policy_list *policies ) const;
    unsigned get_flags( void ) const;

    //  
    bool verify_certificate( const CACMPT_BLOB &cert );
    bool verify_crl( const CACMPT_BLOB &crl );
    bool verify_message( ASN1BEREncodeBuffer &enc_buffer, 
	ASN1C_PKIMessage &message,
	const ASN1TDynBitStr &signature, const CACMPT_BLOB &signer,
	const ASN1T_AlgorithmIdentifier &alg,
	const CACMPT_BLOB &sender_kit );

    //   
    void get_chain( encoded_certificate_list &chain ) const;
    TrustStatus get_status() const;
//    void get_user_policy( external_policy_list &user_policy_set ) const;
//    bool get_user_explicit_policy() const;

    bool toCERT_CHAIN_CONTEXT (void *);

private:
    CertChainBuilder *builder;
    //  
    CertChainContext &operator=( const CertChainContext & );
};

#endif // __CHAIN_UTIL_H
