/*
 * Copyright(C) 2004  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 56176 $
 * \date $Date: 2009-08-08 10:26:13 +0400 (Сбт, 08 Авг 2009) $
 * \author $Author: dim $
 *
 * \brief      
 */

#ifndef _AUDITLOG_H
#define _AUDITLOG_H

#include "common.h"

#include <fstream>

#include "ASN1Blob.h"
#include "CA_CMP_Types.h"

typedef std::ifstream AuditLogInputStream;
typedef std::ofstream AuditLogOutputStream;

// OID  
extern const char sz_id_CryptoPro_audit_service_TSP[];
extern const char sz_id_CryptoPro_audit_service_OCSP[];
extern const char sz_id_CryptoPro_audit_service_DVCS[];
extern const char sz_id_CryptoPro_audit_service_NS[];
// OID  
extern const char sz_id_CryptoPro_audit_record_TSP[];
extern const char sz_id_CryptoPro_audit_record_OCSP[];
extern const char sz_id_CryptoPro_audit_record_DVCS[];
extern const char sz_id_CryptoPro_audit_record_NS[];
// TODO: oids for NS

//   
enum AuditLogVersion
{
    AuditLogVersion_1 = 0
};

/**
 *     
 * (  / )
 */
struct AuditTransportInfoItem
{
    std::wstring name; //<  
    std::wstring value; //<  
};

/**
 *        
 * (  )
 */
struct AuditTransportData
{
    CACMPT_Date time; //<    
    CACMPT_BLOB message; //<  
    std::vector<AuditTransportInfoItem> transportInfo; //<  
};

bool operator==( const AuditTransportData& lhs, const AuditTransportData& rhs);
bool operator!=( const AuditTransportData& lhs, const AuditTransportData& rhs);

/**
 *        (-)
 */
struct AuditTransactionData
{
    AuditTransportData request; //< 
    AuditTransportData response; //< 
};

bool operator==( const AuditTransactionData& lhs, const AuditTransactionData& rhs);
bool operator!=( const AuditTransactionData& lhs, const AuditTransactionData& rhs);

typedef std::vector<AuditTransactionData> AuditRedirectVector;

/**
 *       
 */
struct AuditRecord
{
    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encodedRecord);
public:
    CACMPT_OID type; //<   ( sz_id_CryptoPro_audit_record_TSP)
    AuditTransactionData transaction; //<  
    AuditRedirectVector redirects; //<   
};

bool operator==( const AuditRecord& lhs, const AuditRecord& rhs);
bool operator!=( const AuditRecord& lhs, const AuditRecord& rhs);

enum AuditLog_Status
{
    AuditLog_normal = 0,
    AuditLog_corrupted = 1,
    AuditLog_emergency = 2,
    AuditLog_suspicious = 3,
    AuditLog_corrupted_signedByOther = 4,
    AuditLog_emergency_signedByOther = 5,
    AuditLog_normal_signedByOther = 6
};

/**
 *     
 */
struct AuditLogStatusInfo
{
    AuditLog_Status status;
    std::wstring operatorStatement;
};

class AuditLogImpl;

/**
 *       decode()  
 * AuditLogDecodeContext.       
 *    .  ,   
 *    .
 */
typedef bool (*AuditLogDecodeCallback)(
    const CACMPT_BLOB& encodedRecord,
    void *pvArg);

/**
 *    .
 */
class AuditLogDecodeContext
{
public:
    AuditLogDecodeContext();
    virtual ~AuditLogDecodeContext();

    ///    
    void decode(
	AuditLogInputStream& inStream, 
	AuditLogDecodeCallback callback = 0,
	void* callbackArg = 0);
    ///   
    bool verify( const CACMPT_BLOB& signerCertificate) const;

    //  get
    AuditLogVersion get_version() const;
    const CACMPT_Date& get_startLogTime() const;
    const CACMPT_Date& get_finishLogTime() const;
    const AuditLogStatusInfo& get_statusInfo() const;
    const CACMPT_OID& get_serviceType() const;
    const CACMPT_SignerInfo& get_signerInfo() const;
    const CACMPT_BLOB& get_signerCertificate() const;

    size_t get_recordCount() const;
    AuditRecord get_record( size_t index) const;
private:
    void clear();
    AuditLogImpl* pImpl_;
};

/**
 *    .
 */
class AuditLogEncodeContext
{
public:
    AuditLogEncodeContext(
	const CACMPT_OID& serviceType,
	const CACMPT_Date& startLogTime,
	const CACMPT_Date& finishLogTime); //  
    virtual ~AuditLogEncodeContext();

    ///  
    void start( 
	HCRYPTPROV hCryptProv,
	DWORD dwKeySpec,
	const CACMPT_BLOB& encodedSignerCertificate,
	CACMPT_AlgorithmIdentifier& hashAlgorithm,
	AuditLogOutputStream& outStream);
    ///  
    void append( const AuditRecord& record);
    void append( const unsigned char* pbRecordEncoded, DWORD cbRecordEncoded);
    ///  
    void finish( const AuditLogStatusInfo& statusInfo);

    //  get
    AuditLogVersion get_version() const;
    const CACMPT_Date& get_startLogTime() const;
    const CACMPT_Date& get_finishLogTime() const;
    const CACMPT_OID& get_serviceType() const;
    const AuditLogStatusInfo& get_statusInfo() const;
    const CACMPT_SignerInfo& get_signerInfo() const;
private:
    void clear();
    AuditLogImpl* pImpl_;
};

extern "C" BOOL WINAPI ALEncodeCmsgStreamOutputCallback(
  const void *pvArg,  //in
  BYTE *pbData,       //in
  DWORD cbData,       //in
  BOOL fFinal         //in
);

extern "C" BOOL WINAPI ALDecodeCmsgStreamOutputCallback(
  const void *pvArg,  //in
  BYTE *pbData,       //in
  DWORD cbData,       //in
  BOOL fFinal         //in
);

extern "C" BOOL WINAPI ALDecodeCmsgStreamOutputCallbackNew(
  const void *pvArg,  //in
  BYTE *pbData,       //in
  DWORD cbData,       //in
  BOOL fFinal         //in
);
#endif // _AUDITLOG_H
