/*
 * Copyright(C) 2004  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 63297 $
 * \date $Date: 2010-07-20 11:11:36 +0400 (Втр, 20 Июл 2010) $
 * \author $Author: dim $
 *
 * \brief     DVCS 
 * (Data Validation and Certification Server protocols)
 */

#ifndef _PKIXDVCS_MAIN_H
#define _PKIXDVCS_MAIN_H

#include "common.h"

#include "ASN1Blob.h"
#include "ASN1Types.h"
#include "CA_CMP_Types.h"
#include "CA_CMP_CRL.h"
#include "CA_CMP_Msg.h"
#include "CA_CMP_ErrorMsg.h"
#include "PKIXOCSP_Main.h"

struct DVCS_PathProcInput;
class DVCS_CertEtcToken;
typedef std::vector<DVCS_CertEtcToken> DVCS_CertEtcTokenList;
class DVCS_Time;
struct DVCS_DigestInfo;
class DVCS_TargetEtcChain;
typedef std::vector<DVCS_TargetEtcChain> DVCS_TargetEtcChainList;
class DVCS_RequestInformation;
class DVCS_Data;
class DVCS_CertInfo;
class DVCS_ErrorNotice;

extern const char sz_id_ct_DVCSRequestData[];
extern const char sz_id_ct_DVCSResponseData[];

///     DVCS
enum DVCS_ServiceType
{
    DVCS_ServiceType_cpd = 1,
    DVCS_ServiceType_vsd = 2,
    DVCS_ServiceType_cpkc = 3,
    DVCS_ServiceType_ccpd = 4
};

///  DVCS
enum DVCS_Version
{
    DVCS_Version_1 = 1
};

// messages

/**
 *  DVCS_MessageEncode     
 *  DVCS  (ContentInfo)   
 * DVCS_Request  DVCS_Response.
 */
class DVCS_MessageEncode
{
public:
    DVCS_MessageEncode(	
	const CACMPT_OID& eContentType,
	HCRYPTPROV hCryptProv, 
	DWORD dwKeySpec,
	const CACMPT_BLOB& encodedCertificate,
	const CACMPT_AlgorithmIdentifier hashAlgorithm);
    virtual ~DVCS_MessageEncode();
    CACMPT_BLOB encode();
    void set_encodedContent( const CACMPT_BLOB& encoded);
    void set_certificates( const encoded_certificate_list& certs);
private:
    HCRYPTMSG hMsgEncode_;
};

/**
 *  DVCS_MessageDecode     
 *  DVCS .
 */
class DVCS_MessageDecode
{
public:
    DVCS_MessageDecode();
    virtual ~DVCS_MessageDecode();
    void decode( const CACMPT_BLOB& encoded);
    CACMPT_SignerInfos getSigners() const;
    bool verify( const CACMPT_BLOB& encodedCertificate) const;
    CACMPT_BLOB get_encodedContent() const;
    CACMPT_OID get_innerContentType() const;
    encoded_certificate_list get_certificates() const;
private:
    HCRYPTMSG hMsgDecode_;
};

// request

class DVCS_RequestImpl;

/**
 *  DVCS_Request   ,   DVCS .
 *       
 * DVCS_MessageEncode   DVCS .
 */
class DVCS_Request
{
public:
    DVCS_Request();
    virtual ~DVCS_Request();
    DVCS_Request( 
	const DVCS_RequestInformation& requestInformation,
	const DVCS_Data& data);

    DVCS_Request( const DVCS_Request& src);
    DVCS_Request& operator=( const DVCS_Request& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    const DVCS_RequestInformation& get_requestInformation() const;
    const DVCS_Data& get_data() const;
    const CACMPT_GeneralName* get_transactionIdentifier() const;
    // set
    void set_requestInformation( const DVCS_RequestInformation& requestInformation);
    void set_data( const DVCS_Data& data);
    void set_transactionIdentifier( const CACMPT_GeneralName* transactionIdentifier);
private:
    void clear();
    DVCS_RequestImpl* pImpl_;
};

class DVCS_RequestInformationImpl;

/**
 *  DVCS_Request    
 *  DVCS .
 */
class DVCS_RequestInformation
{
public:
    DVCS_RequestInformation();
    DVCS_RequestInformation( DVCS_ServiceType service);
    virtual ~DVCS_RequestInformation();

    DVCS_RequestInformation( const DVCS_RequestInformation& src);
    DVCS_RequestInformation& operator=( const DVCS_RequestInformation& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    DVCS_Version get_version() const;
    DVCS_ServiceType get_service() const;
    const CACMPT_BigInteger* get_nonce() const;
    const DVCS_Time* get_requestTime() const;
    const CACMPT_GeneralNames* get_requester() const;
    const CACMPT_PolicyInformation* get_requestPolicy() const;
    const CACMPT_GeneralNames* get_dvcs() const;
    const CACMPT_GeneralNames* get_dataLocations() const;
    const CACMPT_Extensions* get_extensions() const;
    // set
    void set_service( DVCS_ServiceType service);
    void set_nonce( const CACMPT_BigInteger* nonce);
    void set_requestTime( const DVCS_Time* requestTime);
    void set_requester( const CACMPT_GeneralNames* requester);
    void set_requestPolicy( const CACMPT_PolicyInformation* requestPolicy);
    void set_dvcs( const CACMPT_GeneralNames* dvcs);
    void set_dataLocations( const CACMPT_GeneralNames* dataLocations);
    void set_extensions( const CACMPT_Extensions* extensions);
private:
    void clear();
    DVCS_RequestInformationImpl* pImpl_;
};

class DVCS_DataImpl;

/**
 *  DVCS_Data   ,   
 *  (Data Validation Certificate).   
 *     (. RFC 3029,  8).
 */
class DVCS_Data
{
public:
    enum Type
    {
	t_message = 1,
	t_messageImprint = 2,
	t_certs = 3
    };
public:
    DVCS_Data();
    virtual ~DVCS_Data();

    DVCS_Data( const DVCS_Data& src);
    DVCS_Data& operator=( const DVCS_Data& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    Type get_type() const;
    //  ,   DVCS_Data   
    //  ,  0.
    // ( get_message()  0 
    // get_type() == t_messageImprint
    const CACMPT_BLOB* get_message() const;
    const DVCS_DigestInfo* get_messageImprint() const;
    const DVCS_TargetEtcChainList* get_certs() const;
    // set (   )
    void set_message( const CACMPT_BLOB& message);
    void set_messageImprint( const DVCS_DigestInfo& messageImprint);
    void set_certs( const DVCS_TargetEtcChainList& certs);
private:
    void clear();
    DVCS_DataImpl* pImpl_;
};

// response

class DVCS_ResponseImpl;

/**
 *  DVCS_Response      DVCS
 *  (DVCS        DVC - Data Validation
 * Certificate).    
 *   DVCS_MessageEncode   DVCS .
 *     (    ).
 */
class DVCS_Response
{
public:
    enum Type
    {
	t_dvCertInfo = 1,
	t_dvErrorNote = 2
    };
public:
    DVCS_Response();
    virtual ~DVCS_Response();

    DVCS_Response( const DVCS_Response& src);
    DVCS_Response& operator=( const DVCS_Response& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    Type get_type() const;
    //  ,   DVCS_Response   
    //  ,  0.
    const DVCS_CertInfo* get_dvCertInfo() const;
    const DVCS_ErrorNotice* get_dvErrorNote() const;
    // set (   )
    void set_dvCertInfo( const DVCS_CertInfo& dvCertInfo);
    void set_dvErrorNote( const DVCS_ErrorNotice& dvErrorNote);
private:
    void clear();
    DVCS_ResponseImpl* pImpl_;
};

class DVCS_CertInfoImpl;

/**
 *  DVCS_CertInfo    
 *  DVCS .
 */
class DVCS_CertInfo
{
public:
    DVCS_CertInfo();
    DVCS_CertInfo(
	const DVCS_RequestInformation& dvReqInfo,
	const DVCS_DigestInfo& messageImprint,
	const CACMPT_BigInteger& serialNumber,
	const DVCS_Time& responseTime);
    virtual ~DVCS_CertInfo();

    DVCS_CertInfo( const DVCS_CertInfo& src);
    DVCS_CertInfo& operator=( const DVCS_CertInfo& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    DVCS_Version get_version() const;
    const DVCS_RequestInformation& get_dvReqInfo() const;
    const DVCS_DigestInfo& get_messageImprint() const;
    const CACMPT_BigInteger& get_serialNumber() const;
    const DVCS_Time& get_responseTime() const;
    const ErrorMsgInfo* get_dvStatus() const;
    const CACMPT_PolicyInformation* get_policy() const;
    const CACMPT_SignerInfos* get_reqSignature() const;
    const DVCS_TargetEtcChainList* get_certs() const;
    const CACMPT_Extensions* get_extensions() const;
    // set
    void set_dvReqInfo( const DVCS_RequestInformation& dvReqInfo);
    void set_messageImprint( const DVCS_DigestInfo& messageImprint);
    void set_serialNumber( const CACMPT_BigInteger& serialNumber);
    void set_responseTime( const DVCS_Time& responseTime);
    void set_dvStatus( const ErrorMsgInfo* dvStatus);
    void set_policy( const CACMPT_PolicyInformation* policy);
    void set_reqSignature( const CACMPT_SignerInfos* reqSignature);
    void set_certs( const DVCS_TargetEtcChainList* certs);
    void set_extensions( const CACMPT_Extensions* extensions);
private:
    void clear();
    DVCS_CertInfoImpl* pImpl_;
};

class DVCS_ErrorNoticeImpl;

/**
 *  DVCS_ErrorNotice    ,
 *      .
 */
class DVCS_ErrorNotice
{
public:
    DVCS_ErrorNotice();
    DVCS_ErrorNotice( const ErrorMsgInfo& transactionStatus);
    virtual ~DVCS_ErrorNotice();

    DVCS_ErrorNotice( const DVCS_ErrorNotice& src);
    DVCS_ErrorNotice& operator=( const DVCS_ErrorNotice& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    const ErrorMsgInfo& get_transactionStatus() const;
    const CACMPT_GeneralName* get_transactionIdentifier() const;
    // set
    void set_transactionStatus( const ErrorMsgInfo& transactionStatus);
    void set_transactionIdentifier( const CACMPT_GeneralName* transactionIdentifier);
private:
    void clear();
    DVCS_ErrorNoticeImpl* pImpl_;
};


// content

/**
 *  DVCS_DigestInfo     .
 */
struct DVCS_DigestInfo
{
    CACMPT_AlgorithmIdentifier digestAlgorithm;
    CACMPT_BLOB digest;

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);
};

class DVCS_TimeImpl;

/**
 *  DVCS_Time        
 * (GeneralizedTime),    .
 */
class DVCS_Time
{
public:
    enum Type
    {
	t_timeStampToken = 1,
	t_genTime = 2
    };
public:
    DVCS_Time();
    virtual ~DVCS_Time();

    DVCS_Time( const DVCS_Time& src);
    DVCS_Time& operator=( const DVCS_Time& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    Type get_type() const;
    //  ,   DVCS_Time   
    //  ,  0.
    const CACMPT_Date* get_genTime() const;
    const CACMPT_BLOB* get_timeStampToken() const;
    // set (   )
    void set_genTime( const CACMPT_Date& genTime);
    void set_timeStampToken( const CACMPT_BLOB& timeStampToken);
private:
    void clear();
    DVCS_TimeImpl* pImpl_;
};

class DVCS_TargetEtcChainImpl;

class DVCS_TargetEtcChain
{
public:
    DVCS_TargetEtcChain();
    DVCS_TargetEtcChain( const DVCS_CertEtcToken& target);
    virtual ~DVCS_TargetEtcChain();

    DVCS_TargetEtcChain( const DVCS_TargetEtcChain& src);
    DVCS_TargetEtcChain& operator=( const DVCS_TargetEtcChain& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    const DVCS_CertEtcToken& get_target() const;
    const DVCS_CertEtcTokenList* get_chain() const;
    const DVCS_PathProcInput* get_pathProcInput() const;
    // set
    void set_target( const DVCS_CertEtcToken& target);
    void set_chain( const DVCS_CertEtcTokenList* chain);
    void set_pathProcInput( const DVCS_PathProcInput* pathProcInput);
private:
    void clear();
    DVCS_TargetEtcChainImpl* pImpl_;
};

struct DVCS_PathProcInput
{
    DVCS_PathProcInput()
    : inhibitPolicyMapping(false),
      explicitPolicyReqd(false),
      inhibitAnyPolicy(false) {}

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    std::vector<CACMPT_PolicyInformation> acceptablePolicySet;
    bool inhibitPolicyMapping;
    bool explicitPolicyReqd;
    bool inhibitAnyPolicy;
};

class DVCS_CertEtcTokenImpl;

class DVCS_CertEtcToken
{
public:
    enum Type
    {
	t_extension       = 1,
	t_certificate     = 2,
	t_pkistatus       = 4,
	t_assertion       = 5,
	t_crl             = 6,
	t_ocspcertstatus  = 7,
	t_oscpcertid      = 8,
	t_oscpresponse    = 9
    };
public:
    DVCS_CertEtcToken();
    virtual ~DVCS_CertEtcToken();

    DVCS_CertEtcToken( const DVCS_CertEtcToken& src);
    DVCS_CertEtcToken& operator=( const DVCS_CertEtcToken& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    // get
    Type get_type() const;
    const CACMPT_Extension* get_extension() const;
    const CACMPT_BLOB* get_certificate() const;
    const ErrorMsgInfo* get_pkistatus() const;
    const CACMPT_BLOB* get_assertion() const;
    const CACMPT_BLOB* get_crl() const;
    const OCSP_CertStatus* get_ocspcertstatus() const;
    const OCSP_CertID* get_oscpcertid() const;
    const OCSP_ResponseMessage* get_oscpresponse() const;
    // set
    void set_extension( const CACMPT_Extension& extension);
    void set_certificate( const CACMPT_BLOB& certificate);
    void set_pkistatus( const ErrorMsgInfo& status);
    void set_assertion( const CACMPT_BLOB& assertion);
    void set_crl( const CACMPT_BLOB& crl);
    void set_ocspcertstatus( const OCSP_CertStatus& certStatus);
    void set_oscpcertid( const OCSP_CertID& certID);
    void set_oscpresponse( const OCSP_ResponseMessage& response);
private:
    void clear();
    DVCS_CertEtcTokenImpl* pImpl_;
};

#define sz_id_aa_ets_certificateRefs "1.2.840.113549.1.9.16.2.21"
#define sz_id_aa_ets_revocationRefs "1.2.840.113549.1.9.16.2.22"
#define sz_id_aa_ets_certValues "1.2.840.113549.1.9.16.2.23"
#define sz_id_aa_ets_revocationValues "1.2.840.113549.1.9.16.2.24"

class DVCS_OcspIdentifier
{
public:
    DVCS_OcspIdentifier();
    DVCS_OcspIdentifier( 
	ResponderIDType ocspResponderIDType,
	const CACMPT_BLOB& ocspResponderID,
	const CACMPT_Date& producedAt);
    ~DVCS_OcspIdentifier();

    DVCS_OcspIdentifier( const DVCS_OcspIdentifier& src);
    DVCS_OcspIdentifier& operator=( const DVCS_OcspIdentifier& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    ResponderIDType get_ocspResponderIDType() const;
    const CACMPT_BLOB& get_ocspResponderID() const;
    const CACMPT_Date& get_producedAt() const;
    //set
    void set_ocspResponderID( ResponderIDType type, const CACMPT_BLOB& ocspResponderID);
    void set_producedAt( const CACMPT_Date& producedAt);
private:
    ResponderIDType type_;
    CACMPT_BLOB ocspResponderID_;
    CACMPT_Date producedAt_;
};

class DVCS_OcspResponsesID
{
public:
    DVCS_OcspResponsesID();
    DVCS_OcspResponsesID( const DVCS_OcspIdentifier& ocspIdentifier);
    ~DVCS_OcspResponsesID();

    DVCS_OcspResponsesID( const DVCS_OcspResponsesID& src);
    DVCS_OcspResponsesID& operator=( const DVCS_OcspResponsesID& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const DVCS_OcspIdentifier& get_ocspIdentifier() const;
    const CACMPT_OtherHash* get_ocspRepHash() const;
    //set
    void set_ocspIdentifier( const DVCS_OcspIdentifier& ocspIdentifier);
    void set_ocspRepHash( const CACMPT_OtherHash* ocspRepHash);
private:
    DVCS_OcspIdentifier ocspIdentifier_;
    std::auto_ptr<CACMPT_OtherHash> ocspRepHash_;
};

class DVCS_OcspListID
{
public:
    DVCS_OcspListID();
    DVCS_OcspListID( const std::list<DVCS_OcspResponsesID>& ocspResponses);
    ~DVCS_OcspListID();

    DVCS_OcspListID( const DVCS_OcspListID& src);
    DVCS_OcspListID& operator=( const DVCS_OcspListID& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const std::list<DVCS_OcspResponsesID>& get_ocspResponses() const;
    //set
    void set_ocspResponses( 
	const std::list<DVCS_OcspResponsesID>& ocspResponses);
private:
    std::list<DVCS_OcspResponsesID> ocspResponses_;
};

class DVCS_CrlIdentifier
{
public:
    DVCS_CrlIdentifier();
    DVCS_CrlIdentifier( const CACMPT_BLOB& crlissuer, const CACMPT_Date& crlIssuedTime);
    ~DVCS_CrlIdentifier();

    DVCS_CrlIdentifier( const DVCS_CrlIdentifier& src);
    DVCS_CrlIdentifier& operator=( const DVCS_CrlIdentifier& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const CACMPT_BLOB& get_crlissuer() const;
    const CACMPT_Date& get_crlIssuedTime() const;
    const int *get_crlNumber() const;
    //set
    void set_crlissuer( const CACMPT_BLOB& crlissuer);
    void set_crlIssuedTime( const CACMPT_Date& crlIssuedTime);
    void set_crlNumber( const int *crlNumber);
private:
    CACMPT_BLOB crlissuer_;
    CACMPT_Date crlIssuedTime_;
    std::auto_ptr<int> crlNumber_;
};

class DVCS_CrlValidatedID
{
public:
    DVCS_CrlValidatedID();
    DVCS_CrlValidatedID( const CACMPT_OtherHash& crlHash);
    ~DVCS_CrlValidatedID();

    DVCS_CrlValidatedID( const DVCS_CrlValidatedID& src);
    DVCS_CrlValidatedID& operator=( const DVCS_CrlValidatedID& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const CACMPT_OtherHash& get_crlHash() const;
    const DVCS_CrlIdentifier* get_crlIdentifier() const;
    //set
    void set_crlHash( const CACMPT_OtherHash& crlHash);
    void set_crlIdentifier( const DVCS_CrlIdentifier* crlIdentifier);
private:
    CACMPT_OtherHash crlHash_;
    std::auto_ptr<DVCS_CrlIdentifier> crlIdentifier_;
};

class DVCS_CRLListID
{
public:
    DVCS_CRLListID();
    DVCS_CRLListID( const std::list<DVCS_CrlValidatedID>& crls);
    ~DVCS_CRLListID();

    DVCS_CRLListID( const DVCS_CRLListID& src);
    DVCS_CRLListID& operator=( const DVCS_CRLListID& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const std::list<DVCS_CrlValidatedID>& get_crls() const;
    //set
    void set_crls( const std::list<DVCS_CrlValidatedID>& crls);
private:
    std::list<DVCS_CrlValidatedID> crls_;
};

class DVCS_CrlOcspRef
{
public:
    DVCS_CrlOcspRef();
    ~DVCS_CrlOcspRef();

    DVCS_CrlOcspRef( const DVCS_CrlOcspRef& src);
    DVCS_CrlOcspRef& operator=( const DVCS_CrlOcspRef& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const DVCS_CRLListID* get_crlids() const;
    const DVCS_OcspListID* get_ocspids() const;
    //set
    void set_crlids( const DVCS_CRLListID* crlids);
    void set_ocspids( const DVCS_OcspListID* ocspids);
private:
    std::auto_ptr<DVCS_CRLListID> crlids_;
    std::auto_ptr<DVCS_OcspListID> ocspids_;
};

class DVCS_CompleteRevocationRefs: public std::list<DVCS_CrlOcspRef>
{
public:
    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);
};

class DVCS_CompleteCertificateRefs: public std::list<CACMPT_OtherCertID>
{
public:
    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);
};

typedef std::list<CACMPT_BLOB> OCSPBasicResponseEncodedList;

class DVCS_RevocationValues
{
public:
    DVCS_RevocationValues();
    ~DVCS_RevocationValues();

    DVCS_RevocationValues( const DVCS_RevocationValues& src);
    DVCS_RevocationValues& operator=( const DVCS_RevocationValues& src);

    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);

    //get
    const encoded_crl_list *get_crlVals() const;
    const OCSPBasicResponseEncodedList *get_ocspVals() const;
    //set
    void set_crlVals( const encoded_crl_list *crlVals);
    void set_ocspVals( const OCSPBasicResponseEncodedList *ocspVals);
private:
    std::auto_ptr<encoded_crl_list> crlVals_;
    std::auto_ptr<OCSPBasicResponseEncodedList> ocspVals_;
};

class DVCS_CertificateValues: public encoded_certificate_list
{
public:
    CACMPT_BLOB encode() const;
    void decode( const CACMPT_BLOB& encoded);
};

#endif // _PKIXDVCS_MAIN_H
