/*
 * Copyright(C) 2004  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 41452 $
 * \date $Date: 2007-07-04 17:43:11 +0400 (Срд, 04 Июл 2007) $
 * \author $Author: dim $
 *
 * \brief     OCSP (Online Certificate Status Protocol)
 */

#ifndef _PKIXOCSP_MAIN_H
#define _PKIXOCSP_MAIN_H

#include "common.h"
#include <list>
#include "ASN1Blob.h"
#include "CA_CMP_Types.h"
#include "CA_CMP_CRL.h"

extern const char sz_id_kp_OCSPSigning[];
extern const char sz_id_pkix_ocsp_nocheck[];

///  OCSP
enum OCSP_Version
{
    OCSP_Version_1 = 0,
    OCSP_Version_2 = 1
};

///   
enum OCSP_ResponseStatus
{
    OCSP_ResponseStatus_successful =	    0,	//Response has valid confirmations
    OCSP_ResponseStatus_malformedRequest =  1,	//Illegal confirmation request
    OCSP_ResponseStatus_internalError =	    2,	//Internal error in issuer
    OCSP_ResponseStatus_tryLater =	    3,	//Try again later
    OCSP_ResponseStatus_sigRequired =	    5,	//Must sign the request
    OCSP_ResponseStatus_unauthorized =	    6,	//Request unauthorized
    OCSP_ResponseStatus_badCRL =	    8	//Error in CRL processing
};

// request

class OCSP_SingleRequest;
typedef std::list<OCSP_SingleRequest> OCSP_SingleRequestList;
class OCSP_ReqCert;

/**
 *  OCSP_RequestMessage      .
 */
class OCSP_RequestMessage
{
public:
    OCSP_RequestMessage();
    ~OCSP_RequestMessage();

    OCSP_RequestMessage( const OCSP_RequestMessage& src);
    OCSP_RequestMessage& operator=( const OCSP_RequestMessage& src);

    ///  
    CACMPT_BLOB encode() const;
    ///  
    void decode( const CACMPT_BLOB& encodedRequest);

    ///  
    void sign(
	HCRYPTPROV hCryptProv,
	DWORD dwKeySpec,
	const CACMPT_BLOB& encodedCert,
	const CACMPT_AlgorithmIdentifier& hashAlgorithm,
	const encoded_certificate_list* additionalCertificates = 0);
    ///    (:    )
    bool verify(
	const CACMPT_BLOB& encodedCertificate) const;

    // get (    )
    OCSP_Version get_version() const;
    const CACMPT_BLOB* get_requestorName() const;
    const OCSP_SingleRequestList* get_requestList() const;
    const CACMPT_Extensions* get_requestExtensions() const;
    //      sign()
    const CACMPT_AlgorithmIdentifier* get_signatureAlgorithm() const;
    const CACMPT_BLOB* get_signature() const;
    const encoded_certificate_list* get_signatureCerts() const;
    // set (  , NULL  )
    void set_requestorName( const CACMPT_BLOB* encodedName);
    void set_requestList( const OCSP_SingleRequestList* requestList);
    void set_requestExtensions( const CACMPT_Extensions* extensions);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

/**
 *  OCSP_SingleRequest       .
 */
class OCSP_SingleRequest
{
public:
    ///    
    OCSP_SingleRequest( const OCSP_ReqCert& reqCert);

    OCSP_SingleRequest( const OCSP_SingleRequest& src);
    OCSP_SingleRequest& operator=( const OCSP_SingleRequest& src );

    ~OCSP_SingleRequest();

    // get (    )
    const OCSP_ReqCert& get_reqCert() const;
    const CACMPT_Extensions* get_singleRequestExtensions() const;
    // set (  , NULL  )
    void set_singleRequestExtensions( const CACMPT_Extensions* extensions);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

struct OCSP_CertID
{
    CACMPT_AlgorithmIdentifier hashAlgorithm;
    CACMPT_BLOB issuerNameHash;
    CACMPT_BLOB issuerKeyHash;
    CACMPT_BigInteger serialNumber;
};

///  OCSP_CertID
OCSP_CertID MakeCertID(
    const CACMPT_AlgorithmIdentifier& hashAlgorithm,
    const CACMPT_BLOB& certificate,
    const CACMPT_BLOB& issuerCertificate);

struct OCSP_CertIdWithSignature
{
    CACMPT_BLOB issuer;
    CACMPT_BigInteger serialNumber;
    CACMPT_BLOB tbsCertificateHash;
    CACMPT_AlgorithmIdentifier signatureAlgorithm;
    CACMPT_BLOB signatureValue;
};

///  OCSP_CertIdWithSignature
OCSP_CertIdWithSignature MakeCertIdWithSignature(
    const CACMPT_BLOB& certificate);

/**
 *  OCSP_RequestMessage   .
 *     : OCSP_CertID, OCSP_CertIdWithSignature
 *      (fullCertificate).  
 *   OCSP.       OCSP_CertID, 
 *   v1,       
 * ,    v2.
 */
class OCSP_ReqCert
{
public:
    ///   -   
    enum fullCertificateType { Certificate, AttributeCertificate };

    //     
    OCSP_ReqCert( const OCSP_CertID& certId);
    OCSP_ReqCert( fullCertificateType type, const CACMPT_BLOB& encodedCertificate);
    OCSP_ReqCert( const OCSP_CertIdWithSignature& certIdWithSignature);

    OCSP_ReqCert( const OCSP_ReqCert& src);
    OCSP_ReqCert& operator=( const OCSP_ReqCert& src);

    ~OCSP_ReqCert();

    //get (    )
    //   get  0   
    //     
    const OCSP_CertID* get_certID() const;
    const CACMPT_BLOB* get_fullCert() const;
    const fullCertificateType* get_fullCertType() const;
    const OCSP_CertIdWithSignature* get_certIdWithSignature() const;
    //set (  )
    void set_certID( const OCSP_CertID& certId);
    void set_fullCert( fullCertificateType type, const CACMPT_BLOB& encodedCertificate);
    void set_certIdWithSignature( const OCSP_CertIdWithSignature& certIdWithSignature);
public:
    class Impl;
private:
    void clear();
    Impl* pImpl_;
};

// response

class OCSP_SingleResponse;
typedef std::list<OCSP_SingleResponse> OCSP_SingleResponseList;
class OCSP_BasicResponse;
class OCSP_CertStatus;

/**
 *  OCSP_ResponseMessage       
 * .
 */
class OCSP_ResponseMessage
{
public:
    // responseStatus   internalError
    OCSP_ResponseMessage();
    ~OCSP_ResponseMessage();

    OCSP_ResponseMessage( const OCSP_ResponseMessage& src);
    OCSP_ResponseMessage& operator=( const OCSP_ResponseMessage& src);

    ///  
    CACMPT_BLOB encode() const;
    ///  
    void decode( const CACMPT_BLOB& encodedResponse);

    // get (    )
    const OCSP_ResponseStatus& get_responseStatus() const;
    const CACMPT_OID* get_responseType() const;
    const CACMPT_BLOB* get_response() const;
    // set 
    void set_responseStatus( const OCSP_ResponseStatus& status);
    // (  , NULL  )
    void set_response(
	const CACMPT_OID* responseType,
	const CACMPT_BLOB* response);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

///    -       
enum ResponderIDType 
{
    responderID_ByName, 
    responderID_ByHash 
};

/**
 *  OCSP_BasicResponse      .
 */
class OCSP_BasicResponse
{
public:
    ///    
    OCSP_BasicResponse( 
	const CACMPT_Date producedAt,
	ResponderIDType type,
	const CACMPT_BLOB &responderID);
    OCSP_BasicResponse();
    ~OCSP_BasicResponse();

    OCSP_BasicResponse( const OCSP_BasicResponse& src);
    OCSP_BasicResponse& operator=( const OCSP_BasicResponse& src);

    /// 
    CACMPT_BLOB encode() const;
    ///  
    void decode( const CACMPT_BLOB& encodedResponse);

    ///  
    void sign(
	HCRYPTPROV hCryptProv,
	DWORD dwKeySpec,
	const CACMPT_BLOB& encodedCert,
	const CACMPT_AlgorithmIdentifier& hashAlgorithm);
    ///   
    bool verify( 
	const CACMPT_BLOB& encodedCertificate) const;

    // get (    )
    OCSP_Version get_version() const;
    const CACMPT_Date& get_producedAt() const;
    ResponderIDType get_responderIDType() const;
    const CACMPT_BLOB& get_responderID() const;
    // signature  signatureAlgorithm   sign()
    const CACMPT_AlgorithmIdentifier* get_signatureAlgorithm() const;
    const CACMPT_BLOB* get_signature() const;
    const OCSP_SingleResponseList* get_responses() const;
    const encoded_certificate_list* get_certs() const;
    const CACMPT_Extensions* get_responseExtensions() const;
    // set (  , NULL  )
    void set_responses( const OCSP_SingleResponseList* responses);
    void set_certs( const encoded_certificate_list* certs);
    void set_responseExtensions( const CACMPT_Extensions* extensions);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

// OID   BasicResponse
extern const char sz_id_pkix_ocsp_basic[];

/**
 *  OCSP_SingleResponse        
 *  .
 */
class OCSP_SingleResponse
{
public:
    ///    
    OCSP_SingleResponse(
	const OCSP_ReqCert& reqCert,
	const OCSP_CertStatus& certStatus,
	const CACMPT_Date& thisUpdate);
    OCSP_SingleResponse( const OCSP_SingleResponse& src);
    OCSP_SingleResponse& operator=( const OCSP_SingleResponse& src);

    ~OCSP_SingleResponse();

    // get (    )
    const OCSP_ReqCert& get_reqCert() const;
    const OCSP_CertStatus& get_certStatus() const;
    const CACMPT_Date& get_thisUpdate() const;
    const CACMPT_Date* get_nextUpdate() const;
    const CACMPT_Extensions* get_singleExtensions() const;
    // set (  , NULL  )
    void set_nextUpdate( const CACMPT_Date* nextUpdate);
    void set_singleExtensions( const CACMPT_Extensions* extensions);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

///   UnknownInfo
enum OCSP_UnknownInfo { ocspUINull = 0xffff };

/**
 *  OCSP_CertStatus   
 */
class OCSP_CertStatus
{
public:
    ///  
    enum Type { Good, Revoked, Unknown };
    
    //     
    //    - unknown
    OCSP_CertStatus();
    OCSP_CertStatus(
	const CACMPT_Date& revocationTime,
	const CrlReason* revocationReason);
    OCSP_CertStatus( OCSP_UnknownInfo info);

    OCSP_CertStatus( const OCSP_CertStatus& src);
    OCSP_CertStatus& operator=( const OCSP_CertStatus& src);

    ~OCSP_CertStatus();

    //get (    )
    bool isGood() const;
    bool isRevoked() const;
    bool isUnknown() const;
    Type get_type() const;
    const CACMPT_Date* get_revocationTime() const;
    const CrlReason* get_revocationReason() const;
    const OCSP_UnknownInfo* get_unknownInfo() const;
    //set (     ,
    //    )
    void set_good();
    void set_revoked(
	const CACMPT_Date& revocationTime,
	const CrlReason* revocationReason);
    void set_unknown( OCSP_UnknownInfo info);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

bool operator==( const OCSP_CertStatus& lhs, const OCSP_CertStatus& rhs);

// 

// CRL Reference

extern const char sz_id_pkix_ocsp_crl[];

class OCSP_CrlIDImpl;

class OCSP_ExtCrlID : public CACMPT_ExtValue
{
public:
    OCSP_ExtCrlID();
    OCSP_ExtCrlID( const CACMPT_BLOB& der);
    ~OCSP_ExtCrlID();

    OCSP_ExtCrlID( const OCSP_ExtCrlID& src);
    OCSP_ExtCrlID& operator=( const OCSP_ExtCrlID& src);

    //get
    const std::string* get_crlUrl() const;
    const DWORD* get_crlNum() const;
    const CACMPT_Date* get_crlTime() const;
    //set
    void set_crlUrl( const std::string* crlUrl);
    void set_crlNum( const DWORD* crlNum);
    void set_crlTime( const CACMPT_Date* crlTime);
private:
    void clear();
    OCSP_CrlIDImpl* pImpl_;
};

// Historical request

extern const char sz_id_CryptoPro_ocsp_historical_request[];

class OCSP_ExtHistoricalRequest : public CACMPT_ExtValue
{
public:
    OCSP_ExtHistoricalRequest();
    OCSP_ExtHistoricalRequest( const CACMPT_BLOB& der);
    ~OCSP_ExtHistoricalRequest();

    OCSP_ExtHistoricalRequest( const OCSP_ExtHistoricalRequest& src);
    OCSP_ExtHistoricalRequest& operator=( const OCSP_ExtHistoricalRequest& src);

    //get
    const std::string* get_crlUrl() const;
    const DWORD* get_crlNum() const;
    const CACMPT_Date* get_crlTime() const;
    //set
    void set_crlUrl( const std::string* crlUrl);
    void set_crlNum( const DWORD* crlNum);
    void set_crlTime( const CACMPT_Date* crlTime);
private:
    OCSP_CrlIDImpl* pImpl_;
};

// CRL Locator

extern const char sz_id_CryptoPro_ocsp_crl_locator[];

class OCSP_ExtCRLLocator: public CACMPT_ExtValue
{
public:
    OCSP_ExtCRLLocator();
    OCSP_ExtCRLLocator( 
	const CACMPT_CRLDistPointsSyntax& distributionPoints);
    OCSP_ExtCRLLocator( const CACMPT_BLOB& der);
    ~OCSP_ExtCRLLocator() {}

    OCSP_ExtCRLLocator( const OCSP_ExtCRLLocator& src);
    OCSP_ExtCRLLocator& operator=( const OCSP_ExtCRLLocator& src);

    void encode();
    void decode();

    //get
    const CACMPT_CRLDistPointsSyntax& get_distributionPoints() const;
    //set
    void set_distributionPoints( 
	const CACMPT_CRLDistPointsSyntax& distributionPoints);
private:
    CACMPT_CRLDistPointsSyntax distributionPoints_;
};

// Nonce

extern const char sz_id_pkix_ocsp_nonce[];

class OCSP_ExtNonce: public CACMPT_ExtValue
{
public:
    OCSP_ExtNonce();
    OCSP_ExtNonce( const CACMPT_BLOB& der);
    ~OCSP_ExtNonce() {}

    OCSP_ExtNonce( const OCSP_ExtNonce& src);
    OCSP_ExtNonce& operator=( const OCSP_ExtNonce& src);

    void encode();
    void decode();

    //get
    const CACMPT_BLOB& get_nonce() const;
    //set
    void set_nonce( const CACMPT_BLOB& nonce);
private:
    CACMPT_BLOB nonce_;
};

// Instant revocation indicator

extern const char sz_id_CryptoPro_ocsp_instant_revocation_indicator[];

class OCSP_ExtInstantRevocationIndicator: public CACMPT_ExtValue
{
public:
    OCSP_ExtInstantRevocationIndicator();
    OCSP_ExtInstantRevocationIndicator( 
	const CACMPT_RevAnnContent& instantRevocationIndicator);
    OCSP_ExtInstantRevocationIndicator( const CACMPT_BLOB& der);
    ~OCSP_ExtInstantRevocationIndicator() {}

    OCSP_ExtInstantRevocationIndicator( 
	const OCSP_ExtInstantRevocationIndicator& src);
    OCSP_ExtInstantRevocationIndicator& operator=(
	const OCSP_ExtInstantRevocationIndicator& src);

    void encode();
    void decode();

    //get
    const CACMPT_RevAnnContent& get_instantRevocationIndicator() const;
    //set
    void set_instantRevocationIndicator( 
	const CACMPT_RevAnnContent& instantRevocationIndicator);
private:
    CACMPT_RevAnnContent instantRevocationIndicator_;
};

// Revocation announcement reference

extern const char sz_id_CryptoPro_ocsp_revocation_announcement_reference[];

class OCSP_ExtRevocationAnnouncementReference: public CACMPT_ExtValue
{
public:
    OCSP_ExtRevocationAnnouncementReference();
    OCSP_ExtRevocationAnnouncementReference(
	const CACMPT_RevAnnContent& revocationAnnouncementReference);
    OCSP_ExtRevocationAnnouncementReference( const CACMPT_BLOB& der);
    ~OCSP_ExtRevocationAnnouncementReference() {}

    OCSP_ExtRevocationAnnouncementReference( 
	const OCSP_ExtRevocationAnnouncementReference& src);
    OCSP_ExtRevocationAnnouncementReference& operator=(
	const OCSP_ExtRevocationAnnouncementReference& src);

    void encode();
    void decode();

    //get
    const CACMPT_RevAnnContent& get_revocationAnnouncementReference() const;
    //set
    void set_revocationAnnouncementReference( 
	const CACMPT_RevAnnContent& revocationAnnouncementReference);
private:
    CACMPT_RevAnnContent revocationAnnouncementReference_;
};

// Service locator

extern const char sz_id_pkix_ocsp_service_locator[];

class OCSP_ExtServiceLocator: public CACMPT_ExtValue
{
public:
    OCSP_ExtServiceLocator();
    OCSP_ExtServiceLocator( const CACMPT_BLOB& der);
    ~OCSP_ExtServiceLocator() {}

    OCSP_ExtServiceLocator( const OCSP_ExtServiceLocator& src);
    OCSP_ExtServiceLocator& operator=( const OCSP_ExtServiceLocator& src);

    void encode();
    void decode();

    //get
    const CACMPT_BLOB& get_issuer() const;
    const CACMPT_AuthorityInfoAccessSyntax* get_locator() const;
    //set
    void set_issuer( const CACMPT_BLOB& issuer);
    void set_locator( const CACMPT_AuthorityInfoAccessSyntax* locator);
private:
    CACMPT_BLOB issuer_;
    std::auto_ptr<CACMPT_AuthorityInfoAccessSyntax> locator_;
};

// Acceptable responses

typedef std::list<CACMPT_OID> OCSP_AcceptableResponses;

extern const char sz_id_pkix_ocsp_response[];

class OCSP_ExtAcceptableResponses: public CACMPT_ExtValue
{
public:
    OCSP_ExtAcceptableResponses();
    OCSP_ExtAcceptableResponses( 
	const OCSP_AcceptableResponses& acceptableResponses);
    OCSP_ExtAcceptableResponses( const CACMPT_BLOB& der);
    ~OCSP_ExtAcceptableResponses() {}

    OCSP_ExtAcceptableResponses( const OCSP_ExtAcceptableResponses& src);
    OCSP_ExtAcceptableResponses& operator=( const OCSP_ExtAcceptableResponses& src);

    void encode();
    void decode();

    //get
    const OCSP_AcceptableResponses& get_acceptableResponses() const;
    //set
    void set_acceptableResponses( 
	const OCSP_AcceptableResponses& acceptableResponses);
private:
    OCSP_AcceptableResponses acceptableResponses_;
};

// Archive Cutoff

extern const char sz_id_pkix_ocsp_archive_cutoff[];

class OCSP_ExtArchiveCutoff: public CACMPT_ExtValue
{
public:
    OCSP_ExtArchiveCutoff();
    OCSP_ExtArchiveCutoff( 
	const CACMPT_Date& archiveCutoff);
    OCSP_ExtArchiveCutoff( const CACMPT_BLOB& der);
    ~OCSP_ExtArchiveCutoff() {}

    OCSP_ExtArchiveCutoff( const OCSP_ExtArchiveCutoff& src);
    OCSP_ExtArchiveCutoff& operator=( const OCSP_ExtArchiveCutoff& src);

    void encode();
    void decode();

    //get
    const CACMPT_Date& get_archiveCutoff() const;
    //set
    void set_archiveCutoff( 
	const CACMPT_Date& archiveCutoff);
private:
    CACMPT_Date archiveCutoff_;
};

#endif // _PKIXOCSP_MAIN_H
