/*
 * Copyright(C) 2004  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 60692 $
 * \date $Date: 2010-03-21 12:12:55 +0300 (Вск, 21 Мар 2010) $
 * \author $Author: dim $
 *
 * \brief     TSP (Time-Stamp Protocol)
 */

#ifndef _PKIXTSP_MAIN_H
#define _PKIXTSP_MAIN_H

#include "common.h"
#include "ASN1Blob.h"
#include "ASN1Types.h"
#include "CA_CMP_Types.h"

class TimeStampToken;
class ErrorMsgInfo;
class CACMPT_Attributes;
struct TSP_Accuracy;
struct CACMPT_SignerInfo;
typedef CACMPT_SignerInfo TSP_SignerInfo;

extern const char sz_id_ct_TSTInfo[];
extern const char sz_id_timestampCountersignature[];
extern const char sz_id_kp_timeStamping[];

/**
 * /mainpage
 *
 *  TSP       
 * .     ,   RFC 3161,   
 *   Microsoft     (Authenticode).
 * 
 *        RFC 3161,  
 * :
 *
 * TimeStampRequest -    
 * TimeStampResponse -      
 * TimeStampToken -  
 *
 *     Microsoft  :
 *
 * TimeStampAuthenticodeRequest -     Microsoft
 * TimeStampAuthenticode -   Microsoft
 */


#if 0
TimeStampReq ::= SEQUENCE  {
   version                  INTEGER  { v1(1) },
   messageImprint           MessageImprint,
     --a hash algorithm OID and the hash value of the data to be
     --time-stamped
   reqPolicy                TSAPolicyId                OPTIONAL,
   nonce                    INTEGER                    OPTIONAL,
   certReq                  BOOLEAN                    DEFAULT FALSE,
   extensions               [0] IMPLICIT Extensions    OPTIONAL  }

MessageImprint ::= SEQUENCE  {
     hashAlgorithm                AlgorithmIdentifier,
     hashedMessage                OCTET STRING  }
#endif //0

/**
 * \brief      
 *
 *  TimeStampRequest      .
 *   ,     der-
 *    .
 */
class TimeStampRequest
{
public:
    /**
     * \brief 
     *       .
     *
     * \param hashAlgorithm [in]  
     * \param hashedMessage [in]      
     *        
     * \param certReq [in] ,     
     *              
     */
    TimeStampRequest(
	const CACMPT_AlgorithmIdentifier& hashAlgorithm,
	const CACMPT_BLOB& hashedMessage,
	bool certReq = false);
    ///   ,    decode()
    TimeStampRequest();
    virtual ~TimeStampRequest();
    //  get   
    //    
    /**
     * \brief   
     *
     * \return  
     */
    DWORD get_version() const;
    /**
     * \brief   ,    .
     *
     * \return  
     */
    const CACMPT_AlgorithmIdentifier& get_hashAlgorithm() const;
    /**
     * \brief   ,     
     *
     * \return  ,   
     */
    const CACMPT_BLOB& get_hashedMessage() const;
    /**
     * \brief    certReq
     *
     * \remarks  certReq   ,    ,
     *          .
     *     false,      ,
     *  true - .
     * \return true  false
     */
    bool get_certReq() const;
    //  get   .
    //     .   ,
    //     .
    /**
     * \brief     .
     *
     * \return   OID. 
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_OID* get_reqPolicy() const;
    /**
     * \brief   nonce    
     *
     * \return   ,    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_BigInteger* get_nonce() const;
    /**
     * \brief     
     * 
     * \return    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_Extensions* get_extensions() const;

    //  set   .
    // E  NULL -    
    //       .
    /**
     * \brief     
     * 
     * \param reqPolicy [in]   OID 
     * 
     * \remarks  .   NULL,   
     *       .
     */
    void set_reqPolicy( const CACMPT_OID* reqPolicy);
    /**
     * \brief   nonce
     * 
     * \param nonce [in]  ,    
     * 
     * \remarks  .   NULL,   
     *       .
     */
    void set_nonce( const CACMPT_BigInteger* nonce);
    /**
     * \brief       
     *
     * \param extensions [in]    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_extensions( const CACMPT_Extensions* extensions); 

    /**
     * \brief   
     *
     * \return      der-
     *		   
     */
    CACMPT_BLOB encode() const;
    /**
     * \brief      der-
     *
     * \param encodedRequest [in]     der-
     *				       
     */
    void decode( const CACMPT_BLOB& encodedRequest);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};
                             
#if 0
TimeStampResp ::= SEQUENCE  {
     status                  PKIStatusInfo,
     timeStampToken          TimeStampToken     OPTIONAL  }
#endif //0

/**
 * \brief        
 *
 *  TimeStampResponse     
 *  .  ,    ,
 *   ,     ,  . 
 * 
 */
class TimeStampResponse
{
public:
    /**
     * \brief   .
     * 
     * \remarks     - PKI_STATUS_GRANTED
     */
    TimeStampResponse();
    virtual ~TimeStampResponse();

    //  get
    /*
     * \brief      
     *
     * \return ,     
     */
    const ErrorMsgInfo& get_status() const;
    /**
     * \brief      
     * 
     * \return    
     * 
     * \remarks  .   NULL,   
     *    .
     */
    const TimeStampToken* get_timeStampToken() const;

    //  set
    /**
     * \brief        
     *
     * \param token [in]    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_timeStampToken( const TimeStampToken* token);
    /**
     * \brief     
     *
     * \param errorInfo [in] ,     
     *
     */
    void set_status( const ErrorMsgInfo& errorInfo);

    /**
     * \brief     
     *
     * \return      der-
     *		     
     */
    CACMPT_BLOB encode() const;
    /**
     * \brief        der-
     *
     * \param encodedRequest [in]     der-
     *				       
     */
    void decode( const CACMPT_BLOB& encodedResponse);
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

#if 0
TimeStampToken ::= ContentInfo

     -- contentType is id-signedData as defined in [CMS]
     -- content is SignedData as defined in([CMS])
     -- eContentType within SignedData is id-ct-TSTInfo
     -- eContent within SignedData is TSTInfo

TSTInfo ::= SEQUENCE  {
    version                      INTEGER  { v1(1) },
    policy                       TSAPolicyId,
    messageImprint               MessageImprint,
      -- MUST have the same value as the similar field in
      -- TimeStampReq
    serialNumber                 INTEGER,
     -- Time-Stamping users MUST be ready to accommodate integers
     -- up to 160 bits.
    genTime                      GeneralizedTime,
    accuracy                     Accuracy                 OPTIONAL,
    ordering                     BOOLEAN             DEFAULT FALSE,
    nonce                        INTEGER                  OPTIONAL,
      -- MUST be present if the similar field was present
      -- in TimeStampReq.  In that case it MUST have the same value.
    tsa                          [0] GeneralName          OPTIONAL,
    extensions                   [1] IMPLICIT Extensions  OPTIONAL   }
#endif //0

/**
 * \brief    .
 *
 *   ,     .
 *      CMS 
 * (ContentInfo  SignerInfo,   eContentType
 *  id-ct-TSTInfo). ,   , 
 *      .
 *
 */
class TimeStampToken
{
public:
    /**
     * \brief 
     * 
     *       .
     *
     * \param TSAPolicyId [in] OID    
     * \param hashAlgorithm [in]  
     * \param hashedMessage [in]      
     *        
     * \param serialNumber [in]     
     * \param genTime [in]     (UTC)
     * \param ordering [in] ,     
     *			       accuracy
     */
    TimeStampToken(
	const CACMPT_OID& policy,
	const CACMPT_AlgorithmIdentifier& hashAlgorithm,
	const CACMPT_BLOB& hashedMessage,
	const CACMPT_BigInteger& serialNumber,
	const CACMPT_Date& genTime,
	bool ordering = false);
    ///   ,    decode()
    TimeStampToken();
    ///  
    TimeStampToken( const TimeStampToken& src);

    ~TimeStampToken();
    //  get   
    //     
    /**
     * \brief   
     *
     * \return  
     */
    DWORD get_version() const;
    /**
     * \brief      
     *
     * \return OID  
     */
    const CACMPT_OID& get_policy() const;
    /**
     * \brief   ,    .
     *
     * \return  
     */
    const CACMPT_AlgorithmIdentifier& get_hashAlgorithm() const;
    /**
     * \brief   ,     
     *
     * \return  ,   
     */
    const CACMPT_BLOB& get_hashedMessage() const;
    /**
     * \brief     
     *
     * \return ,    
     */
    const CACMPT_BigInteger& get_serialNumber() const;
    /**
     * \brief    
     *
     * \return   
     */
    const CACMPT_Date& get_genTime() const;
    /**
     * \brief    ordering
     *
     * \return true  false
     */
    bool get_ordering() const;
    /**
     * \brief   SignerInfo (  )
     *
     * \return ,     (SignerInfo, RFC 3161)
     */
    const TSP_SignerInfo& get_signerInfo() const;
    //  get   .
    //     .   ,
    //     .
    /**
     * \brief  ,     
     *
     * \return   ,    
     *		     
     *    
     * \remarks  .   NULL,   
     *    .
     */
    const TSP_Accuracy* get_accuracy() const;
    /**
     * \brief   nonce    
     *
     * \return   ,    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_BigInteger* get_nonce() const;
    /**
     * \brief     
     * 
     * \return    ,  
     *		der-    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_BLOB* get_tsa() const;
    /**
     * \brief  ,   
     *
     * \return    ,    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_Extensions* get_extensions() const;
    /**
     * \brief   ,    
     *
     * \return    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_Attributes* get_signedAttributes() const;
    /**
     * \brief   ,    
     *
     * \return    
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_Attributes* get_unsignedAttributes() const;
    /**
     * \brief      
     *
     * \return     
     *
     * \remarks  .   NULL,   
     *    .
     */
    const encoded_certificate_list* get_certificates() const;

    //  set   .
    // E  NULL -    
    //       .
    /**
     * \brief    accuracy (    )
     *
     * \param accuracy [in]   ,    
     *		     
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_accuracy( const TSP_Accuracy* accuracy);
    /**
     * \brief   nonce
     *
     * \param nonce [in] ,    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_nonce( const CACMPT_BigInteger* nonce);
    /**
     * \brief     ,  
     *
     * \param tsa [in]    ,  
     *		der-    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_tsa( const CACMPT_BLOB* tsa);
    /**
     * \brief     
     *
     * \param extensions [in]    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_extensions( const CACMPT_Extensions* extensions);
    /**
     * \brief      
     *
     * \param attributes [in]    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_signedAttributes( const CACMPT_Attributes* attributes);
    /**
     * \brief      
     *
     * \param attributes [in]    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_unsignedAttributes( const CACMPT_Attributes* attributes);
    /**
     * \brief      
     *
     * \param certificateList [in]   
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_certificates( const encoded_certificate_list* certificateList);

    /**
     * \brief    
     *
     * \return      der-
     *		 
     */
    CACMPT_BLOB encode() const;
    /**
     * \brief       der-
     *
     * \param encodedRequest [in]     der-
     *				   
     */
    void decode( const CACMPT_BLOB& encodedTSToken);
    /**
     * \brief   
     *
     * \param hCryptProv [in]  ,     
     * \param dwKeySpec [in]  
     * \param encodedCert [in]   
     * \param hashAlgorithm [in] ,      
     *
     * \remarks        , 
     *		  
     */
    void sign(
	HCRYPTPROV hCryptProv,
	DWORD dwKeySpec,
	const CACMPT_BLOB& encodedCert,
	const CACMPT_AlgorithmIdentifier& hashAlgorithm);
    ///      
    /**
     * \brief    
     * 
     * \param encodedTSACertificate [in] ,    
     *
     * \return true,    ,  false
     */
    bool verify( const CACMPT_BLOB& encodedTSACertificate) const;
private:
    void clear();
    class Impl;
    Impl* pImpl_;
};

#if 0
Accuracy ::= SEQUENCE {
                seconds        INTEGER           OPTIONAL,
                millis     [0] INTEGER  (1..999) OPTIONAL,
                micros     [1] INTEGER  (1..999) OPTIONAL  }
#endif //0

/**
 * \brief      .
 *
 *      ,  
 *      .
 */
struct TSP_Accuracy
{
    /**
     * \brief 
     *
     * \param seconds_ [in]   
     * \param millis_ [in]   
     * \param micros_ [in]   
     */
    TSP_Accuracy( unsigned seconds_ = 0, unsigned millis_ = 0, unsigned micros_ = 0)
	: seconds(seconds_), millis(millis_), micros(micros_) {}
    ///  
    TSP_Accuracy( const TSP_Accuracy& src)
	: seconds(src.seconds), millis(src.millis), micros(src.micros) {}
    ///  
    TSP_Accuracy& operator=( const TSP_Accuracy& src)
    { seconds = src.seconds; millis = src.millis; micros = src.micros; return *this; }

    unsigned seconds; //< 
    unsigned millis; //< 
    unsigned micros; //< 
};

class TimeStampAuthenticodeRequestImpl;

#if 0
TimeStampAuthenticodeRequest ::= SEQUENCE {
	timeStampAlgorithm		OBJECT IDENTIFIER,
	attributes				SEQUENCE OF Attribute OPTIONAL,
	contentInfo				ContentInfo   }
#endif //0

/**
 * \brief        Authenticode (Microsoft)
 *
 *    ,       
 * Authenticode
 *
 */
class TimeStampAuthenticodeRequest
{
public:
    ///   
    TimeStampAuthenticodeRequest();
    /**
     * \brief 
     *
     * \param timeStampAlgorithm [in] OID   
     * \param contentInfo [in]  ContentInfo   
     *				  
     */
    TimeStampAuthenticodeRequest(
	const CACMPT_OID& timeStampAlgorithm,
	const CACMPT_ContentInfo& contentInfo);
    virtual ~TimeStampAuthenticodeRequest();

    /**
     * \brief       Authenticode
     *
     * \return      der-
     *		    Authenticode
     */
    CACMPT_BLOB encode() const;
    /**
     * \brief      Authenticode     der-
     *
     * \param encodedRequest [in]     der-
     *				      Authenticode
     */
    void decode( const CACMPT_BLOB& encoded);

    //get
    /**
     * \brief  OID'    Authenticode
     *
     * \return  OID
     */
    const CACMPT_OID& get_timeStampAlgorithm() const;
    /**
     * \brief      
     *
     * \return  ContentInfo,  ,    
     */
    const CACMPT_ContentInfo& get_contentInfo() const;
    /**
     * \brief  ,       Authenticode
     *
     * \return    ,   
     *
     * \remarks  .   NULL,   
     *    .
     */
    const CACMPT_Attributes* get_attributes() const;
    //set
    /**
     * \brief     Authenticode
     *
     * \param timeStampAlgorithm [in]  OID
     */
    void set_timeStampAlgorithm( const CACMPT_OID& timeStampAlgorithm);
    /**
     * \brief      
     *
     * \param contentInfo [in]  ContentInfo,  , 
     *				   
     */
    void set_contentInfo( const CACMPT_ContentInfo& contentInfo);
    /**
     * \brief    
     *
     * \param attributes [in]    
     *
     * \remarks  .   NULL,   
     *       .
     */
    void set_attributes( const CACMPT_Attributes* attributes);
private:
    void clear();
    TimeStampAuthenticodeRequestImpl* pImpl_;
};

class TimeStampAuthenticodeImpl;

/**
 *  TimeStampAuthenticodeRequest 
 *   Authenticode.
 */
class TimeStampAuthenticode
{
public:
    TimeStampAuthenticode();
    TimeStampAuthenticode( 
	const CACMPT_BLOB& content,
	const CACMPT_Date& signingTime );
    virtual ~TimeStampAuthenticode();

    /// 
    CACMPT_BLOB encode() const;
    /// 
    void decode( const CACMPT_BLOB& encoded);

    /// 
    void sign(
	HCRYPTPROV hCryptProv,
	DWORD dwKeySpec,
	const CACMPT_BLOB& certificate,
	const CACMPT_AlgorithmIdentifier& hashAlgorithm);
    ///  
    bool verify( const CACMPT_BLOB& certificate) const;

    // get
    const CACMPT_BLOB& get_content() const;
    const CACMPT_Date& get_signingTime() const;
    CACMPT_SignerInfo get_signerInfo() const;
    const encoded_certificate_list* get_certificates() const;
    // set
    void set_content( const CACMPT_BLOB& content) const;
    void set_signingTime( const CACMPT_Date& signingTime);
    void set_certificates( const encoded_certificate_list* certificates);
private:
    void clear();
    TimeStampAuthenticodeImpl* pImpl_;
};


#endif // _PKIXTSP_MAIN_H
