/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile: CA_CMP_CRL.h,v $
 * \version $Revision: 1.18 $
 * \date $Date: 2005/01/24 12:36:43 $
 * \author $Author: cav $
 *
 * \brief   
 */

#ifndef CA_CMP_CRL_H
#define CA_CMP_CRL_H

#include "CA_CMP_Types.h"

enum CrlReason {
    CRLREASON_UNSPECIFIED = 0,
    CRLREASON_KEYCOMPROMISE = 1, 
    CRLREASON_CACOMPROMISE = 2, 
    CRLREASON_AFFILIATIONCHANGED = 3,
    CRLREASON_SUPERSEDED = 4, 
    CRLREASON_CESSATIONOFOPERATION = 5, 
    CRLREASON_CERTIFICATEHOLD = 6, 
    CRLREASON_REMOVEFROMCRL = 8,
    CRLREASON_PRIVILEDGEWITHDRAWN = 9,
    CRLREASON_AACOMPROMISE = 10
};

enum HoldInstruction {
    HOLDINSTRUCTION_UNSPECIFIED = 0,
    HOLDINSTRUCTION_NONE = 1,
    HOLDINSTRUCTION_CALLISSUER = 2,
    HOLDINSTRUCTION_REJECT = 3
};

std::string CrlReason_toString( CrlReason );
CrlReason CrlReason_fromString( const char* );

std::string HoldInstruction_toString( HoldInstruction );
HoldInstruction HoldInstruction_fromString( const char* );

std::string HoldInstruction_toOid( HoldInstruction );
HoldInstruction HoldInstruction_fromOid( const char* );

class CrlInfoItem
{
public:
    CrlInfoItem();
    CrlInfoItem( const CrlInfoItem &item ) { assign( item ); }
    CrlInfoItem( const char* Serial, const CACMPT_Date Date = CACMPT_Date() );
    ~CrlInfoItem();
    CrlInfoItem& operator =( const CrlInfoItem &right )
    { assign( right ); return *this; }
    bool operator ==( const CrlInfoItem &right ) const
    { return !strcmp( SerialNumber, right.SerialNumber ); }

    //  CrlReason  extenion
    //  CRLREASON_UNSPECIFIED,  Extension 
    CrlReason GetReason() const;
    //  CrlReason  extenion
    //  CRLREASON_UNSPECIFIED Extension 
    void SetReason( CrlReason );

    //  HoldInstruction  extenion
    //  HOLDINSTRUCTION_UNSPECIFIED,  Extension 
    HoldInstruction GetHoldInstruction() const;
    //  HoldInstruction  extenion
    //  HOLDINSTRUCTION_UNSPECIFIED Extension 
    void SetHoldInstruction( HoldInstruction );

    //  InvalidityDate  extenion
    //  CACMPT_Date(),  Extension 
    CACMPT_Date GetInvalidityDate() const;
    //  InvalidityDate  extenion
    //  invalidity_date == CACMPT_Date() Extension 
    void SetInvalidityDate( const CACMPT_Date &invalidity_date );

    //  
    CACMPT_Date RevokedWhen;

    //  
    CACMPT_SERIAL SerialNumber;

    // extensions
    CACMPT_Extensions Extensions;

protected:
    void assign( const CrlInfoItem &right );
};

class CrlInfo : public std::list<CrlInfoItem>
{
public:
    CrlInfo();
    ~CrlInfo();

    CrlInfo( const CrlInfo& crl ): std::list<CrlInfoItem>()
    {
	*this = crl;
    }
    CrlInfo& operator=( const CrlInfo& crl );
    void clear();
    
    // binary blob
    CACMPT_BLOB RawCRL;
    
    // thisUpdate
    CACMPT_Date ThisUpdate;
    
    // nextUpdate
    CACMPT_Date NextUpdate;

    // Use ThisUpdate & NextUpdate
    bool UseUpdate;

    //  - ASN1T_AlgorithmIdentifier signatureAlgorithm;
    char CrlSignatureAlgorithm[ 254 ];
    
    int Status;

    int RequestID;

    CACMPT_PARSED_RDN issuerRDN;

    void remove( const char* serial );

    //   
    CACMPT_BLOB SubjectKeyIdentifier;

    //   
    CACMPT_BLOB AuthorityKeyIdentifier;

    // extensions
    CACMPT_Extensions Extensions;
};

extern const char* sz_id_ce_reasonCode;

class CACMPT_ExtCRLReason: public CACMPT_ExtValue
{
public:
    CACMPT_ExtCRLReason();
    CACMPT_ExtCRLReason( 
	const CrlReason& reason);
    CACMPT_ExtCRLReason( const CACMPT_BLOB& der);

    ~CACMPT_ExtCRLReason() {}

    void encode();
    void decode();

    //get
    const CrlReason& get_reason() const;
    //set
    void set_reason( 
	const CrlReason& reason);
private:
    CrlReason reason_;
};

extern const char* sz_id_ce_instructionCode;

// possible code oids
extern const char* sz_id_holdinstruction_none;
extern const char* sz_id_holdinstruction_callissuer;
extern const char* sz_id_holdinstruction_reject;

class CACMPT_ExtHoldInstructionCode: public CACMPT_ExtValue
{
public:
    CACMPT_ExtHoldInstructionCode();
    CACMPT_ExtHoldInstructionCode( 
	const CACMPT_OID& code);
    CACMPT_ExtHoldInstructionCode( const CACMPT_BLOB& der);

    ~CACMPT_ExtHoldInstructionCode() {}

    void encode();
    void decode();

    //get
    const CACMPT_OID& get_code() const;
    //set
    void set_code( 
	const CACMPT_OID& code);
private:
    CACMPT_OID code_;
};

#endif // CA_CMP_CRL_H
