/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile: PKIXCMP_Server.h,v $
 * \version $Revision: 1.70.2.1 $
 * \date $Date: 2007/05/10 10:53:42 $
 * \author $Author: alexl $
 *
 * \brief PKIXCMP_Server class
 */

#ifndef PKIXCMP_SERVER_H
#define PKIXCMP_SERVER_H

#include <utility>
#ifndef _WIN32
#include <pthread.h>
#endif

#include "PKIXCMP_Base.h"
#include "WinCryptEx.h"
#ifndef _WIN32
#define BUGGED_SERVICE 1
#endif
#ifdef BUGGED_SERVICE
class KAThread;
#endif
class PKIXCMP_Server : public PKIXCMP_Base {
public:
    PKIXCMP_Server( const char *ini_path, WndProv *wnd_prov );
    PKIXCMP_Server( const char *szProvider=0, DWORD dwProvType = PROV_GOST_2001_DH );
    ~PKIXCMP_Server();

    //    .
    // dn [in] : RDN  .
    // szContainer [in/out] :   .
    //        , , , 
    //     "\\\\.\\HDIMAGE\\"; 
    static PKIXCMP_Server *GenerateRoot( const char *ini_path,
	const wchar_t *dn, 
	const CACMPT_Extensions& Extensions,
	const wchar_t *caname,
	const char *szProvider = 0,
	int dwProvType = PROV_GOST_2001_DH,
	const char *szContainer = 0,
	const CACMPT_Period &cert_period = CACMPT_Period::OneYear,
	const char *cdp = 0,
	const CACMPT_Period &crl_period = CACMPT_Period::OneMonth,
	bool boInstallToRootStorage = false,
	WndProv *wnd_prov = 0 );
    //    .
    // dn [in] : RDN  .
    // szContainer [in/out] :   .
    //        , , , 
    //     "\\\\.\\HDIMAGE\\"; 

    static PKIXCMP_Server *GenerateRootCDPS( const char *ini_path,
	const wchar_t *dn, 
	const CACMPT_Extensions& Extensions,
	const wchar_t *caname,
	const char *szProvider = 0,
	int dwProvType = PROV_GOST_2001_DH,
	const char *szContainer = 0,
	const CACMPT_Period &cert_period = CACMPT_Period::OneYear,
	const std::vector<std::string> & cdps = std::vector<std::string>(),
	const CACMPT_Period &crl_period = CACMPT_Period::OneMonth,
	bool boInstallToRootStorage = false,
	WndProv *wnd_prov = 0 );
    
    //     
    // ann_msg [out] :     
    // ann_size [in/out] :  ann_msg
    // szContainer [in/out] :    .
    //        , , , 
    //     "\\\\.\\HDIMAGE\\"; 

    int UpdateRoot( const char *ini_path,
	const char * szProvider, int dwProvType, 
        const char *szContainer, CACMPT_BLOB &ann_msg,
	PKIXCMP_Message *info, WndProv *wnd_prov = 0,bool ToSign=true ,
	const CACMPT_Period *new_date_period = NULL, CACMPT_BLOB *new_cert = NULL,
	const CACMPT_Extensions* extensions = NULL, const char *cdp = NULL);
    //     
    // ann_msg [out] :     
    // ann_size [in/out] :  ann_msg
    // szContainer [in/out] :    .
    //        , , , 
    //     "\\\\.\\HDIMAGE\\"; 

    int UpdateRootCDPS( const char *ini_path,
	const char * szProvider, int dwProvType, 
        const char *szContainer, CACMPT_BLOB &ann_msg,
	PKIXCMP_Message *info, WndProv *wnd_prov = 0,bool ToSign=true ,
	const CACMPT_Period *new_date_period = NULL, CACMPT_BLOB *new_cert = NULL,
	const CACMPT_Extensions* extensions = NULL, const std::vector<std::string> & cdps = std::vector<std::string>(), bool replace_extensions = false );

    int ChangePassword( WndProv *wnd_prov );

    //  
    // crl_msg [out] : 
    // crl_size [in/out] :  crl_msg
    // nextUpdateHours [in] :    ( )
    int GenerateCRL (CACMPT_BLOB &crl_msg, const CrlInfo *info, 
	PKIXCMP_Message *crlAnninfo = NULL,bool ToSign=true);

    //       
    // Request   .
    int CertReqMessage_PreProcess( const CACMPT_BLOB &cr_msg,
	CertificateInfo *info );

    //       
    //    .   Self-Issued
    // 
    int CertReqCert_PreProcess( const CACMPT_BLOB &cert,
	CertificateInfo *info );

    //   (   ),
    //   (      
    // SOAP )
    // change_info ,      
    //   ,     .
    int CertReqMessage_Process(CertificateInfo *info,
	CACMPT_BLOB &cp_msg, int nCertId = -1,bool ToSign=true);

    //      SOAP .
    int CertRepMessage_Process(void *cp_msg, int cp_size);

    int CertReqMessage_Reject( RequestInfo &request,
	CACMPT_BLOB &cp_msg, bool toSign );

    const CACMPT_BLOB& GetCert () const { return MYCert; }

    //     .

    int RevAnnContent_Process( CACMPT_BLOB &ra_msg,
	const RevAnnContentInfo *info,bool ToSign=true );

    unsigned long GetCSP() const { return hProv; }

    CACMPT_BLOB GetCrl();

    //    cross .
    int CrossReqMessage_EncodeAndSign( const CertificateInfo *info,
	CACMPT_BLOB &cr_msg,bool ToSign=true );
    
    int ErrorMsgInfo_EncodeAndSign( const ErrorMsgInfo *em_info,
	CACMPT_BLOB &em_msg, bool ToSign=true )
    { return
	PKIXCMP_Base::ErrorMsgInfo_EncodeAndSign( em_info, em_msg, 
	hProv, AT_SIGNATURE, MYCert, ToSign );
    }
#ifdef LEGACY_TID
    void SetTransactionID( const CACMPT_BLOB* ptid ) 
    { 
	if ( ptid )
	    m_transactionID = *ptid; 
	else m_transactionID.clear();
    }
    const CACMPT_BLOB& GetTransactionID() const { return m_transactionID; }

    void SetMsgNonce( const CACMPT_BLOB* pnonce ) 
    { 
	if ( pnonce )
	    m_msgNonce = *pnonce; 
	else m_msgNonce.clear();
    }
    const CACMPT_BLOB& GetMsgNonce() const { return m_msgNonce; }
#endif
    int CertAnnContent_EncodeAndSign( CertificateInfo *info,
	CACMPT_BLOB &ann_msg,bool ToSign=true);

    //       Root
    void BuildChain( encoded_certificate_list &path )
    { CertChain_VerifyCertificate( GetCert(), path, 0, ADD_STORE, 
	ADD_STORE_LEN ); }

    //      Body
    int GenMsgContent_EncodeAndSign( 
	PKIXCMP_Message *info, CACMPT_BLOB &msg,bool ToSign=true);

    //      Body,  
    int GenMsgContent_Encode(
	PKIXCMP_Message *info, CACMPT_BLOB &msg );

protected:
    PKIXCMP_Server (CACMPT_BLOB* cert, WndProv *wnd_prov,
	const char * szProvider, DWORD dwProvType );
    int CertReqMessage_Process(CertificateInfo *info,
	CACMPT_BLOB &cp_msg, CACMPT_SERIAL szSerial /* in/out */, 
	int nCertId = -1,bool ToSign=true);
    int GenerateRoot (const wchar_t *dn, 
	const CACMPT_Extensions& Extensions,
	const char * szProvider,
	int dwProvType,
	const char *szContainer,
	const CACMPT_Period &period = CACMPT_Period::OneYear,
	const std::vector<std::string> & cdps = std::vector<std::string> (), bool boInstallToRootStorage = false, WndProv *wnd_prov = 0 );
    int CertReqMessage_PreProcess( const CACMPT_BLOB &cr_msg,
	CertificateInfo *info, int dummy );
    int CertReqCert_PreProcess( const CACMPT_BLOB &cert,
	CertificateInfo *info, int dummy );
    int GenerateCRL (CACMPT_BLOB &crl_msg, const CrlInfo *info,
	const CACMPT_Period &period, PKIXCMP_Message *crlAnnInfo = NULL,bool ToSign=true );

    void *hProvParts;
    void init (const CACMPT_BLOB& cert, WndProv *wnd_prov );
    void release ();
    void init ( const char * szProvider, DWORD dwProvType );

    std::string ini_path;
#ifdef LEGACY_TID
    CACMPT_BLOB m_msgNonce;
    CACMPT_BLOB m_transactionID;
    CACMPT_BLOB m_kid;
#endif

#ifdef BUGGED_SERVICE
    KAThread * KA;
#endif    
public:
    static HCRYPTPROV create_container_root( WndProv *wnd_prov, 
	const char *container, const char *provider, DWORD provtype,
	void* &parts, HCRYPTPROV hProvVer );
    static void read_container_root( HCRYPTPROV hProvCA,
	WndProv *wnd_prov, void *&parts );
    static void close_container_root( HCRYPTPROV hProvCA, void *&parts );

public:
    static const CertificateStore *ADD_STORE;
    static const int ADD_STORE_LEN;

    virtual const CertificateStore *Get_ADD_STORE () const { return ADD_STORE; }
    virtual int Get_ADD_STORE_LEN () const { return ADD_STORE_LEN; }
};
#endif // PKIXCMP_SERVER_H
