/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 68026 $
 * \date $Date: 2011-05-15 21:56:42 +0400 (Вск, 15 Май 2011) $
 * \author $Author: dim $
 *
 * \brief   
 */

#ifndef CA_CMP_CRL_H
#define CA_CMP_CRL_H

#include "CA_CMP_Types.h"

enum CrlReason {
    CRLREASON_UNSPECIFIED = 0,
    CRLREASON_KEYCOMPROMISE = 1, 
    CRLREASON_CACOMPROMISE = 2, 
    CRLREASON_AFFILIATIONCHANGED = 3,
    CRLREASON_SUPERSEDED = 4, 
    CRLREASON_CESSATIONOFOPERATION = 5, 
    CRLREASON_CERTIFICATEHOLD = 6, 
    CRLREASON_REMOVEFROMCRL = 8,
    CRLREASON_PRIVILEDGEWITHDRAWN = 9,
    CRLREASON_AACOMPROMISE = 10
};

enum HoldInstruction {
    HOLDINSTRUCTION_UNSPECIFIED = 0,
    HOLDINSTRUCTION_NONE = 1,
    HOLDINSTRUCTION_CALLISSUER = 2,
    HOLDINSTRUCTION_REJECT = 3
};

CrlReason CrlReason_fromString( const char* );
HoldInstruction HoldInstruction_fromString( const char* );
HoldInstruction HoldInstruction_fromOid( const char* );

class CrlInfoItem
{
public:
    CrlInfoItem() {*SerialNumber = 0;}
    CrlInfoItem( const CrlInfoItem &item ) { assign( item ); }
    CrlInfoItem( const char* Serial, const CACMPT_Date Date = CACMPT_Date() );
    ~CrlInfoItem() {}
    bool operator ==( const CrlInfoItem &right ) const
        { return !strcmp( SerialNumber, right.SerialNumber ); }
 
    //  CrlReason  extenion
    //  CRLREASON_UNSPECIFIED,  Extension 
    CrlReason GetReason() const;
    //  CrlReason  extenion
    //  CRLREASON_UNSPECIFIED Extension 
    void SetReason( CrlReason );

    //  HoldInstruction  extenion
    //  HOLDINSTRUCTION_UNSPECIFIED,  Extension 
    HoldInstruction GetHoldInstruction() const;
    //  HoldInstruction  extenion
    //  HOLDINSTRUCTION_UNSPECIFIED Extension 
    void SetHoldInstruction( HoldInstruction );

    //  InvalidityDate  extenion
    //  CACMPT_Date(),  Extension 
    CACMPT_Date GetInvalidityDate() const;
    //  InvalidityDate  extenion
    //  invalidity_date == CACMPT_Date() Extension 
    void SetInvalidityDate( const CACMPT_Date &invalidity_date );

    //  
    CACMPT_Date RevokedWhen;

    //  
    CACMPT_SERIAL SerialNumber;

    // extensions
    CACMPT_Extensions Extensions;

protected:
    void assign( const CrlInfoItem &right );

private:
    CrlInfoItem& operator =( const CrlInfoItem &right );
};

class CrlInfo : public std::list<CrlInfoItem>
{
public:
    CrlInfo();
    ~CrlInfo() {}

    void clear();

    // binary blob
    CACMPT_BLOB RawCRL;

    // thisUpdate
    CACMPT_Date ThisUpdate;

    // nextUpdate
    CACMPT_Date NextUpdate;

    // Use ThisUpdate & NextUpdate
    bool UseUpdate;

    //  - ASN1T_AlgorithmIdentifier signatureAlgorithm;
    char CrlSignatureAlgorithm[ 254 ];

    CACMPT_PARSED_RDN issuerRDN;

    //   
    CACMPT_BLOB SubjectKeyIdentifier;

    //   
    CACMPT_BLOB AuthorityKeyIdentifier;

    // extensions
    CACMPT_Extensions Extensions;
  private:
    CrlInfo(const CrlInfo& crl);
    CrlInfo& operator=( const CrlInfo& crl );
};
#endif // CA_CMP_CRL_H
