/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

#if !defined( _READER_SUPPORT_SUPPORT_FKC_H )
#define _READER_SUPPORT_SUPPORT_FKC_H

#include "common.h"
#include "reader/support.h"
#include "reader/dprint.h"
#if defined UNIX
#   include <stdlib.h>
#else
#   include <malloc.h>
#endif /* UNIX */

#if defined UNIX
#   define _malloca malloc
#   define _freea free
#   include <wchar.h>
#   include "CSP_WinDef.h"
#   include "ISO-IEC_TR_24731-1.h"
#endif  /* UNIX */

#define SUPFKC_ASN_OK            0      /* normal completion status             */
#define SUPFKC_ASN_E_ENDOFBUF   -2      /* unexpected end of buffer on decode   */
#define SUPFKC_ASN_E_IDNOTFOU   -3      /* identifer not found                  */
#define SUPFKC_ASN_E_INVOBJID   -4      /* invalid object identifier            */
#define SUPFKC_ASN_E_INVLEN     -5      /* invalid field length                 */
#define SUPFKC_ASN_E_NOMEM      -12     /* no dynamic memory available          */
#define SUPFKC_ASN_E_BADTAG     -21     /* invalid tag value                    */
#define SUPFKC_ASN_E_INVUTF8    -26     /* invalid UTF-8 encoding               */
#define SUPFKC_ASN_E_INVPARAM   -30     /* invalid parameter                    */

#define SUPFKC_MAX_WCHAR_LEN	1024    /*  ,     UTF8String*/

#ifdef UNICODE
#   define safe_2unilen( src ) (wcslen (src) + 1)
#   define safe_uni2len( src ) ((wcslen (src) + 1) * sizeof(wchar_t))

#   define safe_2unincpy( dest, src, destlen, srclen ) wcsncpy( dest, src, srclen?destlen:destlen )
#   define safe_2unicpy( dest, src ) wcscpy( dest, src )
#   define safe_uni2ncpy( dest, src, destlen, srclen ) wcsncpy( dest, src, destlen?srclen:srclen )
#   define safe_uni2cpy( dest, src ) wcscpy( dest, src )
#else /* UNICODE */
#   define safe_2unilen( src ) MultiByteToWideChar( CP_ACP, 0, (src), (int)(-1), NULL, (int)(0))
#   define safe_uni2len( src ) WideCharToMultiByte( CP_ACP, 0, (src), (int)(-1), NULL, (int)(0), NULL, NULL )

#   define safe_2unincpy( dest, src, destlen, srclen ) MultiByteToWideChar( CP_ACP, 0, (src), (int)(srclen), (dest), (int)(destlen) )
#   define safe_2unicpy( dest, src ) safe_2unincpy(dest, src, (strlen(src)+1), ((strlen(src)+1)*sizeof(wchar_t)))
#   define safe_uni2ncpy( dest, src, destlen, srclen ) WideCharToMultiByte( CP_ACP, 0, (src), (int)(srclen), (dest), (int)(destlen), NULL, NULL )
#   define safe_uni2cpy( dest, src ) safe_uni2ncpy( dest, src, ((wcslen(src)+1)*sizeof(wchar_t)), (wcslen(src)+1))
#endif /* UNICODE */


/* src -   NULL .     wchar_t *,     */
#define safe_utf8len_inwc( src ) (MultiByteToWideChar( CP_UTF8, 0, (src), (int)(-1), NULL, (int)(0) ))

/* src -   NULL . srclen -     .
       wchar_t *,     */
#define safe_utf8len_inwc_n( src, srclen ) (MultiByteToWideChar( CP_UTF8, 0, (src), (int)(srclen), NULL, (int)(0) ))

/*   UTF8   wchar_t *
    dest -     wchar_t * , 
    src -   BYTE *, 
    destlen -   dest  wchar_t, 
    srclen -   src    
    */
#define safe_utf8stowcs( dest, src, destlen, srclen) MultiByteToWideChar( CP_UTF8, 0, (src), (int)(srclen), (dest), (int)(destlen) )

/* src -   wchar_t .      UTF8 . */
#define safe_wcslen_inutf8( src ) (WideCharToMultiByte( CP_UTF8, 0, (src), (int)(-1), NULL, (int)(0), NULL, NULL ))

/* src -   wchar_t . srclen -  src  wchar_t. 
        UTF8 . */
#define safe_wcslen_inutf8_n( src, srclen ) (WideCharToMultiByte( CP_UTF8, 0, (src), (int)(srclen), NULL, (int)(0), NULL, NULL ))

/*   wchar_t   UTF8
    dest -     BYTE *, 
    src -   wchar_t * , 
    destlen -   dest   , 
    srclen -   src  wchar_t
    */
#define safe_wcstoutf8s( dest, src, destlen, srclen ) WideCharToMultiByte( CP_UTF8, 0, (src), (int)(srclen), (dest), (int)(destlen), NULL, NULL )

/*       */
static SUP_INLINE int _utf8_support_gettaglen (int len) 
{
/* SUPFKC_MAX_WCHAR_LEN=1024 */
    int taglen = 4;
    if (len < 0 || len > SUPFKC_MAX_WCHAR_LEN ) taglen = SUPFKC_ASN_E_INVLEN;
    else if (len < 128) taglen = 2;
    else if (len < 256) taglen = 3;
/*    else if (len < 256*256) taglen = 4; */
/*    else if (len < 256*256*256) taglen = 5; */
/*    else taglen = 6; */
    return taglen;
}


/*! 
 * \ingroup rdr_ext_fkc_macro
 * \brief      UTF8String  ,  TCHAR* .
 * \param src [in]   TCHAR* .
 * \param srclen [in]     
 * \ret  utf8   \0.  NULL  0.     -   .
 * \sa supfkc_2utf8sncpy, supfkc_2utf8scpy, supfkc_2utf8sfulllen, supfkc_utf8s2ncpy, supfkc_utf8sis0.
 * \req_suprdr
 */
static SUP_INLINE int supfkc_2utf8slen( TCHAR * src, int srclen )
{
    wchar_t * tmpbuf = NULL;
    int len;

    if (!src) return 0;
    tmpbuf = (wchar_t*)_malloca(srclen*sizeof(wchar_t));
    safe_2unincpy (tmpbuf, src, srclen, srclen);

    len = safe_wcslen_inutf8_n (tmpbuf, wcsnlen(tmpbuf, srclen));
    _freea(tmpbuf);
    return len;
}

/*! 
 * \ingroup rdr_ext_fkc_macro
 * \brief      UTF8String,  TCHAR* .
 * \param src [in]   TCHAR* .
 * \param srclen [in]     
 * \ret  UTF8String.  NULL   NULL .    -   .
 * \sa supfkc_2utf8sncpy, supfkc_2utf8scpy, supfkc_2utf8slen, supfkc_utf8s2ncpy, supfkc_utf8sis0.
 * \req_suprdr
 */
static SUP_INLINE int supfkc_2utf8sfulllen( TCHAR * src, int srclen  ) 
{
    int len = supfkc_2utf8slen( src, srclen );
    return (len < 0)?(len):(len + _utf8_support_gettaglen(len));
}


/*    UTF8String */
static SUP_INLINE int _utf8_support_maketag (BYTE * dest, int len)
{
    int taglen;
    taglen = _utf8_support_gettaglen(len);
    if (taglen < 0) return taglen;
    dest[0] = 0xC;
    if (taglen < 3) dest[1] = (BYTE)(len&0x7F);
    else {
	dest[1] = (BYTE) (0x80 | (taglen - 2));
	if (taglen == 3) {
	    dest[2] = (BYTE)(len & 0xff);
	} 
	else if (taglen == 4) {
	    dest[2] = (BYTE)((len & 0xff00)>>8);
	    dest[3] = (BYTE)(len & 0xff);
	} 
#if 0
_utf8_support_gettaglen() can not return 5, 6
	else if (taglen == 5) {
	    dest[2] = (BYTE)((len & 0xff0000)>>16);
	    dest[3] = (BYTE)((len & 0xff00)>>8);
	    dest[4] = (BYTE)(len & 0xff);
	} 
	else if (taglen == 6) {
	    dest[2] = (BYTE)((len & 0x7f0000)>>24);
	    dest[3] = (BYTE)((len & 0xff0000)>>16);
	    dest[4] = (BYTE)((len & 0xff00)>>8);
	    dest[5] = (BYTE)(len & 0xff);
	}
#endif /* 0 */
    }
    return 0;
}

/*    UTF8String */
static SUP_INLINE int _gettag_utf8_support_len (BYTE * src, int srclen)
{
    int lenlen, len;
    BYTE * pos;

    if (srclen < 2) return SUPFKC_ASN_E_INVLEN;
    if (src[0]!= 0xC) return SUPFKC_ASN_E_INVOBJID;
    if (src[1] < 128) return src[1];
    lenlen = src[1] & 0x7F;
    if ((lenlen+2) > srclen) return SUPFKC_ASN_E_INVLEN;
    len = 0;
    pos = src + 2;
    while (lenlen) {
	len <<= 8;
	len |= pos[0];
	lenlen--;
    }
    return len;
}

/*    UTF8String */
static SUP_INLINE int _gettag_tag_support_len (BYTE * src, int srclen)
{
    if (srclen < 2) return SUPFKC_ASN_E_INVLEN;
    if (src[0]!= 0xC) return SUPFKC_ASN_E_INVOBJID;
    if (src[1] < 128) return 2;
    else return ((src[1]&0x7F) + 2);
}


/*! 
 * \ingroup rdr_ext_fkc_macro
 * \brief    TCHAR *,  0,  UTF8String.    srclen .
 *    NULL   ,    ,  NULL-.
 * \param dest [in/out]       ,  UTF8String.
 * \param destlen [in]    UTF8String  .
 * \param src [in]   TCHAR* .
 * \param srclen [in]    src    sizeof(TCHAR) . 
 * \ret   ,   ,   .
 * \sa supfkc_2utf8scpy, supfkc_2utf8slen, supfkc_2utf8sfulllen, supfkc_utf8s2ncpy, supfkc_utf8sis0.
 * \req_suprdr
 */
static SUP_INLINE int supfkc_2utf8sncpy (BYTE * dest, int destlen, TCHAR * src, int srclen)
{
    int destleninside,taglen,tmpsrclen;
    wchar_t * tmpbuf  = NULL; 
    int err = 0;
    
    if (!src) 
    {
	if (destlen < 2 || !dest) return SUPFKC_ASN_E_INVPARAM;
	((BYTE*)dest)[0] = 0x5; ((BYTE*)dest)[1] = (BYTE)0;
	return 2;
    }

    destleninside = supfkc_2utf8slen(src, srclen);
    if (destleninside < 0) return destleninside;
    taglen = _utf8_support_gettaglen (destleninside);
    if (taglen < 0) return taglen;
    if ((taglen + destleninside) > destlen) return SUPFKC_ASN_E_INVLEN;

    tmpbuf  = (wchar_t*)_malloca(srclen*sizeof(wchar_t));
    safe_2unincpy (tmpbuf, src, srclen, srclen);

    err = _utf8_support_maketag(dest, destleninside);
    if(err < 0)  goto done;

    tmpsrclen = (int)wcsnlen(tmpbuf, srclen);
    err = safe_wcstoutf8s((LPSTR)(dest+taglen), tmpbuf, destlen-taglen, tmpsrclen);
    if (err < 0 ) goto done;
    err = destleninside + taglen;
done: 
    _freea(tmpbuf);
    return err;
}

/*! 
 * \ingroup rdr_ext_fkc_macro
 * \brief    TCHAR *,  0,  UTF8String.
 *    NULL   ,    ,  NULL-.
 * \param dest [in/out]       ,  UTF8String.
 * \param destlen [in]    UTF8String  .
 * \param src [in]   TCHAR* .
 * \ret   ,   ,   .
 * \sa supfkc_2utf8sncpy, supfkc_2utf8slen, supfkc_2utf8sfulllen, supfkc_utf8s2ncpy, supfkc_utf8sis0.
 * \req_suprdr
 */
static SUP_INLINE int supfkc_2utf8scpy (BYTE * dest, int destlen, TCHAR * src)
{
    return supfkc_2utf8sncpy (dest, destlen, src, (int)((src)?(_tcslen(src)+1):0));
}


/*! 
 * \ingroup rdr_ext_fkc_macro
 * \brief   TRUE,   UTF8String    (ber-, 
 *   NULL-).
 * \param src [in]      ber-.
 * \param srclen [in]    
 * \ret 1,  NULL-, 0 .      ,  SUPFKC_ASN_E_INVLEN.
 * \sa 2utf8sncpy, 2utf8scpy, 2utf8slen, 2utf8sfulllen, utf8s2ncpy
 * \req_suprdr
 */
static SUP_INLINE int supfkc_utf8sis0 (BYTE * src, int srclen)
{
    if (srclen < 2) return SUPFKC_ASN_E_INVLEN;
    return ((src[0]==0x5) && (src[1] == 0))?1:0;
}


/*! 
 * \ingroup rdr_ext_fkc_macro
 * \brief   UTF8String  TCHAR * ,  0.    srclen .
 *    NULL-,       \ref supfkc_utf8sis0
 * \param dest [out]   TCHAR *    .
 * \param destlen [in]    TCHAR* - ,   ,    sizeof(TCHAR).
 * \param src [in]   UTF8String (ber-,   ).
 * \param srclen [in]    .      . 
 * \ret   ,   ,    .
 * \sa 2utf8sncpy, 2utf8scpy, 2utf8slen, 2utf8sfulllen, utf8s2cpy, utf8sis0.
 * \req_suprdr
 */
static SUP_INLINE int supfkc_utf8s2ncpy (TCHAR * dest, int destlen, BYTE * src, int srclen)
{
    int utf8stringlen, taglen, isnull, desttmp;
    int err = 0;
    wchar_t * tmpbuf  = NULL;
    BYTE * tmpsrc = NULL;

    if (!src || srclen < 2) {
	err = SUPFKC_ASN_E_INVLEN;
	goto done;
    }
    isnull = supfkc_utf8sis0 (src, srclen);
    if (isnull == 1){
	err = SUPFKC_ASN_E_INVOBJID;
	goto done;
    } else if (isnull) {
	err = isnull;
	goto done;
    }
    taglen = _gettag_tag_support_len(src, srclen);
    if (taglen < 0) {
	err = taglen;
	goto done;
    }
    if (srclen < taglen) {
	err = SUPFKC_ASN_E_INVLEN;
	goto done;
    }
    utf8stringlen = _gettag_utf8_support_len (src, srclen);
    if (utf8stringlen < 0) {
	err = utf8stringlen;
	goto done;
    }
    if (srclen < taglen + utf8stringlen) {
	err = SUPFKC_ASN_E_INVLEN;
	goto done;
    }
    tmpbuf = (wchar_t*)_malloca((destlen)*sizeof(wchar_t));
    tmpsrc = (BYTE*)_malloca(utf8stringlen+1);
    memcpy(tmpsrc,src+taglen,utf8stringlen);
    tmpsrc[utf8stringlen]=0;
    
    desttmp = safe_utf8stowcs (tmpbuf, (LPCSTR)(tmpsrc), destlen, utf8stringlen+1);
    if (!desttmp) {
	err = SUPFKC_ASN_E_INVLEN;
	goto done;
    }
    safe_uni2ncpy(dest, tmpbuf, destlen, desttmp);
    err = (int)_tcsnlen(dest, destlen);
done: 
    _freea (tmpsrc);
    _freea (tmpbuf);
    return err;
}

    //
    //           
    //

/*! 
 * \ingroup rdr_ext_fkc_macro
 * \def P2CH(ChildType,Parent)
 * \brief    -  -.   
 *    ,        .
 *     ,      ,
 *     .
 * \param ChildType [in] -,      . 
 * \param Parent [in]    -. 
 * \ret    -.
 * \sa CONSTP2CH, CH2P, CONSTCH2P.
 * \req_suprdr
 */
#define P2CH(ChildType,Parent)	    (Child##ChildType(Parent))

/*! 
 * \ingroup rdr_ext_fkc_macro
 * \def CONSTP2CH(ChildType,Parent)
 * \brief  -  -  -.   
 *    ,        .
 *     ,      ,
 *     .
 * \param ChildType [in] -,      . 
 * \param Parent [in]  -  -. 
 * \ret    -.
 * \sa P2CH, CH2P, CONSTCH2P.
 * \req_suprdr
 */
#define CONSTP2CH(ChildType,Parent) (CChild##ChildType(Parent))

/*! 
 * \ingroup rdr_ext_fkc_macro
 * \def CH2P(ChildType,Child)
 * \brief    -  -.   
 *          .
 *      ,     
 *   ,    .
 * \param ChildType [in] -  ,     -.
 * \param Parent [in]    -. 
 * \ret    -.
 * \sa P2CH, CONSTP2CH, CONSTCH2P.
 * \req_suprdr
 */
#define CH2P(ChildType,Child)	    (Parent##ChildType(Child))

/*! 
 * \ingroup rdr_ext_fkc_macro
 * \def CONSTCH2P(ChildType,Child)
 * \brief  -  -  -.   
 *          .
 *      ,     
 *   ,    .
 * \param ChildType [in] -  ,     -.
 * \param Parent [in]  -  -. 
 * \ret    -.
 * \sa P2CH, CONSTP2CH, CH2P.
 * \req_suprdr
 */
#define CONSTCH2P(ChildType,Child)  (CParent##ChildType(Child))

    // TODO:   ""  WinCrypt
#define MCF_CHECK_size_of(p, type)					\
	((p)->size_of == sizeof(type) ? (p) : NULL)
#define MCF_NO_CHECK_size_of(p, type)	(p)

#define MAKE_CONVERSION_FUNCTIONS(ChildType,ParentType,ChkChild)	\
    static SUP_INLINE							\
    ChildType * Child##ChildType (ParentType * Parent)			\
    {									\
	return (ChildType *)(void *)Parent;				\
    }									\
									\
    static SUP_INLINE							\
    const ChildType * CChild##ChildType (const ParentType * Parent)	\
    {									\
	return (const ChildType *)(const void *)Parent;			\
    }									\
									\
    static SUP_INLINE							\
    ParentType * Parent##ChildType (ChildType * Child)			\
    {									\
	return (ParentType *)(void *)ChkChild(Child, ChildType);	\
    }									\
									\
    static SUP_INLINE							\
    const ParentType * CParent##ChildType (const ChildType * Child)	\
    {									\
	return (const ParentType *)(const void *)ChkChild(Child,	\
						const ChildType);	\
    }


#endif /* !defined( _READER_SUPPORT_SUPPORT_FKC_H ) */
/*+ end of file:$Id: support_fkc.h 108784 2014-07-09 12:36:07Z sagafyin $ +*/
