/* vim:set sw=4 ts=8 fileencoding=cp1251:::WINDOWS-1251[] */
#ifdef _WIN32
    #pragma setlocale("rus")
#endif
/*
 * Copyright(C) 2013-2014  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 *
 *  ,    , 
 *         
 *   .
 *
 *  -   
 *     .
 */

/*!
 * \version $Revision: 89611 $
 * \date $Date: 2013-05-29 23:43:11 +0400 (ср, 29 май 2013) $
 * \author $Author: lse $
 *
 * \brief    TLS   Linux
 */
#ifdef CSP_DRIVER 
typedef __WCHAR_TYPE__ wchar_t;
#endif
#include"KTLS.h"
#include"wincspc.h"
#include "WSKernelPart.h"
#ifndef CSP_DRIVER
#include"WSMemSTDC.h"
#else
#include"drtktls_srv_io.h"
#endif

#if defined(KRN_EMUL)
	// Userspace Example with libcpcdrv_emul.a and libktls_emul.a
    #include <sys/socket.h>
    #include <errno.h>
    #include <string.h>
    #include <stdarg.h>
    #include <stdio.h>
    #include <errno.h>

    static ssize_t
    emulate_recv(int socket, void *buffer, size_t length, int flags)
    {
	ssize_t ret;
	do
	{
	    ret = recv(socket, buffer, length, flags);
	    ret = (ret == -1)?-errno:ret;
	    if (ret == -EINTR || ret == -EAGAIN)
	    {
		struct timespec tvi,tvo;
		tvi.tv_sec=0;
		tvi.tv_nsec=1000;
		nanosleep(&tvi,&tvo);
		continue;
	    }
	} while(0);    
	return ret;
    }
    static ssize_t
    emulate_send(int socket, const void *buffer, size_t length, int flags)
    {
	ssize_t ret;
	do
	{
	    ret = send(socket, buffer, length, flags);
	    ret = (ret == -1)?-errno:ret;
	    if (ret == -EINTR || ret == -EAGAIN)
	    {
		struct timespec tvi,tvo;
		tvi.tv_sec=0;
		tvi.tv_nsec=1000;
		nanosleep(&tvi,&tvo);
		continue;
	    }
	} while(0);    
	return ret;
    }
#else
#ifdef CSP_DRIVER
    #include <linux/kernel.h>
    #include <linux/string.h>
    #include <linux/errno.h>
    #define mark_alloced()
    #define check_alloced()
#endif    
    ssize_t
    emulate_recv(int socket, 
		void *buffer, 	// buffer in kernel space
		size_t length, 
		int flags);
    ssize_t
    emulate_send(int socket, 
		const void *buffer, 	// buffer in kernel space
		size_t length, 
		int flags);
#endif

void * CPCAlloc(LPCPC_CONFIG CSPConfig, size_t dwSize, DWORD dwMemPoolId)
{
    unsigned long dwThreadId;
    void * res=NULL;
    if (CSPConfig == NULL)
	return NULL;
    if (CSPConfig->pArena == NULL)
	return NULL;
    if (CSPConfig->logConfig.get_thread_id == NULL)
	dwThreadId=0;
    else
	dwThreadId=CSPConfig->logConfig.get_thread_id();
    if (CSPConfig->pArena->pAllocMemory(
			CSPConfig->pArena,
			dwSize,
			dwMemPoolId,
			dwThreadId,
			&res)!=S_OK)
	return NULL;
    memset(res,0,dwSize);
    return res;
}

/*      ,   */
void CPCFree(LPCPC_CONFIG CSPConfig, void * ptr, DWORD dwMemPoolId)
{
    if(ptr)
    {
	CSPConfig->pArena->pFreeMemory(CSPConfig->pArena,ptr,dwMemPoolId);
    }
}

#define PRINTF_SIZE 200	      
void CPC_printf(LPCPC_CONFIG pConfig, const char * fmt, ...)
{
    char * buffer=NULL;
    va_list ap;
    va_start(ap,fmt);
    buffer=(char *)CPCAlloc(pConfig,PRINTF_SIZE,MP_WORK);
    vsnprintf(buffer,PRINTF_SIZE-1,fmt,ap);
    va_end(ap);
    pConfig->logConfig.cprint_str(pConfig->logConfig.lpArg,buffer);
    CPCFree(pConfig, buffer, MP_WORK);
}

static char hexchar(int x)
{
    if (x<0 || x>15)
	return '*';
    if (x<10)
	return '0'+x;
    return x-10+'A';	
}

void CPC_hexdump(LPCPC_CONFIG pConfig, const char * msg, void * src, size_t size)
{
    char * buffer=NULL,*ptr=NULL;
    size_t len=strlen(msg);
    size_t sz=len+3*size+size%26+2;
    size_t i;
    unsigned char * s=(unsigned char *)src;
    buffer=(char *)CPCAlloc(pConfig,sz,MP_WORK);
    memcpy(buffer,msg,len);
    ptr=buffer+len;
    for (i=0;i<size;i++)
    {
	*ptr++=hexchar(s[i]>>4);
	*ptr++=hexchar(s[i]&0xF);
	*ptr++=(i%26==25)?'\n':' ';
    }
    *ptr=0;
    pConfig->logConfig.cprint_str(pConfig->logConfig.lpArg,buffer);
    CPCFree(pConfig, buffer, MP_WORK);
}

struct _DRTKTLS_DRV_
{
    CPC_CONFIG Config;
    HCRYPTMODULE hCSP;
    ktls_gost_handle hTLS;
    HCRYPTPROV hProv;
    PRIVKEY Priv; 
    unsigned tlsdb_len;
    unsigned char * our_tlsdb;
};

struct _DRTKTLS_CTX_ 
{    
    PDRTKTLS_DRV pDrv;
    PUBKEY Pub; 
    CPKTLS_HANDLE hCtx; 
};

static VTABLEPROVSTRUC VTABLE_2001 = { 3, 0, 0, PROV_GOST_2001_DH, 0, 0, 0 };

SECURITY_STATUS CAPI_EXTC kinit_gost(PDRTKTLS_DRV pDrv, unsigned * pSize)
{
    ktls_gost_in gi;
    ktls_gost_out go;

    if (pDrv == NULL)
    {
	*pSize = sizeof(DRTKTLS_DRV);
	return STATUS_SUCCESS;
    }
    if (*pSize < sizeof(DRTKTLS_DRV))
    {
	*pSize = sizeof(DRTKTLS_DRV);
	return STATUS_NO_MEMORY;
    }
    memset(pDrv, 0, sizeof(DRTKTLS_DRV));

    pDrv->Config.cbSize = sizeof(CPC_CONFIG);
    if (CPCGetDefaultConfig(&pDrv->Config, NULL))
	return STATUS_INTERNAL_ERROR;

    stdInitMemory(&pDrv->Config.pArena, NULL, 0);
    CPCSetLocks(&pDrv->Config);
    pDrv->Config.logConfig.level = 15;
    if (CPCCreateProvider(&pDrv->hCSP, &pDrv->Config))
	return STATUS_INTERNAL_ERROR;

    memset(&gi, 0, sizeof(gi));
    gi.pConfig = &pDrv->Config;
    gi.hCSP = pDrv->hCSP;
    cpktls_init_gost(NULL, &gi, 0, &go);
    gi.allocatedMem = go.requiredMem;
    pDrv->hTLS = (ktls_gost_handle)CPCAlloc(&pDrv->Config, gi.allocatedMem, MP_WORK);
    cpktls_init_gost(pDrv->hTLS, &gi, 0, &go);

    if (pDrv->hCSP->AcquireContext(pDrv->hCSP, &pDrv->hProv, NULL, CRYPT_VERIFYCONTEXT, &VTABLE_2001))
	return STATUS_INTERNAL_ERROR;
    pDrv->our_tlsdb = NULL;
    return STATUS_SUCCESS;
}

SECURITY_STATUS CAPI_EXTC kset_rng(PDRTKTLS_DRV pDrv, unsigned char * pbData, unsigned cbData)
{
    DWORD ret;
    CRYPT_DATA_BLOB cdb;
    cdb.pbData=pbData;
    cdb.cbData=cbData;
    ret = pDrv->hCSP->SetProvParam(pDrv->hCSP,pDrv->hProv,PP_RANDOM,(BYTE *)&cdb,0);
    return (ret==0)?STATUS_SUCCESS:STATUS_INVALID_PARAMETER_1;
}

SECURITY_STATUS CAPI_EXTC kcreate_SA(PDRTKTLS_DRV pDrv)
{
    CPC_printf(&pDrv->Config,"kcreate_SA called\n");
    if (pDrv->hTLS->tlsdb.CreateEphemFn(pDrv->hCSP,pDrv->hProv,0,&pDrv->Priv,0,&pDrv->tlsdb_len))
	return STATUS_INTERNAL_ERROR;
    CPC_printf(&pDrv->Config,"kcreate_SA ephemfn size determined:%d\n",pDrv->tlsdb_len);
    pDrv->our_tlsdb=(unsigned char *)CPCAlloc(&pDrv->Config,pDrv->tlsdb_len,MP_WORK);
    CPC_printf(&pDrv->Config,"kcreate_SA ptr alloced:%p\n",pDrv->our_tlsdb);
    if (pDrv->hTLS->tlsdb.CreateEphemFn(pDrv->hCSP,pDrv->hProv,0,&pDrv->Priv,pDrv->our_tlsdb,&pDrv->tlsdb_len))
	return STATUS_INTERNAL_ERROR;
    CPC_printf(&pDrv->Config,"kcreate_SA CreateEphomFn ok:\n");
    return STATUS_SUCCESS;
}


SECURITY_STATUS CAPI_EXTC kinit_ctx(PDRTKTLS_DRV pDrv,PDRTKTLS_CTX pDCtx, unsigned * pSize)
{
    if (pDCtx == NULL)
    {
	*pSize=sizeof(DRTKTLS_CTX);
	return STATUS_SUCCESS;
    }
    if (*pSize < sizeof(DRTKTLS_CTX))
    {
	*pSize=sizeof(DRTKTLS_CTX);
	return STATUS_NO_MEMORY;
    }
    memset(pDCtx,0,sizeof(DRTKTLS_CTX));
    pDCtx->pDrv=pDrv;
    return STATUS_SUCCESS;
}

SECURITY_STATUS CAPI_EXTC kdone_ctx(PDRTKTLS_CTX pDCtx)
{
    if (pDCtx->hCtx && pDCtx->pDrv->hTLS->DeleteContext(pDCtx->pDrv->hTLS,0,pDCtx->hCtx))
	return STATUS_INTERNAL_ERROR;
    check_alloced();	

    return STATUS_SUCCESS;
}

SECURITY_STATUS CAPI_EXTC kdone_gost(PDRTKTLS_DRV pDrv)
{

    cpktls_shutdown_gost(pDrv->hTLS, 0);
    pDrv->hCSP->ReleaseContext(pDrv->hCSP, pDrv->hProv, 0);
    CPCFree(&pDrv->Config, pDrv->our_tlsdb, MP_WORK);
    pDrv->our_tlsdb = 0;
    CPCFree(&pDrv->Config, pDrv->hTLS, MP_WORK);
    pDrv->hTLS = 0;
    if(pDrv->hCSP && pDrv->hCSP->DestroyProvider)
    {
    	pDrv->hCSP->DestroyProvider(pDrv->hCSP);
	pDrv->hCSP = 0;
    }
    if(pDrv->Config.pArena->pDoneMemory)
    {
	pDrv->Config.pArena->pDoneMemory(pDrv->Config.pArena);
    }
    return STATUS_SUCCESS;
}

SECURITY_STATUS CAPI_EXTC kexport_SA(PDRTKTLS_DRV pDrv,unsigned char * pbBuffer,unsigned * puLen)
{
    if (pbBuffer == 0)
    {
	*puLen=pDrv->tlsdb_len;
	return STATUS_SUCCESS;
    }
    else if (*puLen < pDrv->tlsdb_len)
    {
	*puLen= pDrv->tlsdb_len;
	return STATUS_NO_MEMORY;
    }
    memcpy(pbBuffer,pDrv->our_tlsdb,pDrv->tlsdb_len);
    *puLen=pDrv->tlsdb_len;
    return STATUS_SUCCESS;
}

SECURITY_STATUS CAPI_EXTC kimport_SA(PDRTKTLS_CTX pCtx, unsigned char * pbSA,unsigned cbSA)
{
    if (pCtx->pDrv->hTLS->tlsdb.deSerializePubKeyFn(pCtx->pDrv->hCSP,pCtx->pDrv->hProv,pbSA,cbSA,0,&pCtx->Pub,&pCtx->pDrv->Config))
	return STATUS_INTERNAL_ERROR;
    return STATUS_SUCCESS;
}


SECURITY_STATUS CAPI_EXTC kimport_keys(PDRTKTLS_CTX pDCtx,void * pvExpDCtx, unsigned cbExpDCtx)
{
    SecBuffer SB;
    SB.pvBuffer=pvExpDCtx;
    SB.cbBuffer=cbExpDCtx;
    SB.BufferType=SECBUFFER_DATA;
    mark_alloced();
    if (pDCtx->pDrv->hTLS->ImportContext(pDCtx->pDrv->hTLS,&SB,&pDCtx->pDrv->Priv,&pDCtx->Pub,0,&pDCtx->hCtx))
	return STATUS_INTERNAL_ERROR;
    return STATUS_SUCCESS;	
}

SECURITY_STATUS CAPI_EXTC kexport_keys(PDRTKTLS_CTX pDCtx, void * pvExpDCtx, unsigned *pcbExpDCtx)
{
    SecBuffer SB;
    DWORD dwErr;

    SB.pvBuffer=pvExpDCtx;
    SB.cbBuffer=*pcbExpDCtx;
    SB.BufferType=SECBUFFER_DATA;
    dwErr = pDCtx->pDrv->hTLS->ExportContext(pDCtx->pDrv->hTLS, pDCtx->hCtx,
					&pDCtx->pDrv->Priv, &pDCtx->Pub,
					CPKTLS_CONTEXT_EXPORT_TO_USER|
					CPKTLS_CONTEXT_EXPORT_DELETE_OLD,
					&SB);
    *pcbExpDCtx=SB.cbBuffer;
    return STATUS_SUCCESS;
}
//-------------------------------------------------------------
// ,     .

#define My_min(a,b) (((a)<(b))?(a):(b))
SECURITY_STATUS CAPI_EXTC kwritefile(PDRTKTLS_CTX pDCtx, int Socket, const void * pvBuffer, unsigned cbBuffer)
{
    //
    //       .
    //
    SECURITY_STATUS scRet;
    SecPkgContext_StreamSizes Sizes;
    SecBufferDesc   Message;
    SecBuffer       Buffers[4];
#if 0    
    SecBuffer *     pDataBuffer;
    SecBuffer *     pExtraBuffer;
    SecBuffer       ExtraBuffer;
    DWORD cbData=0;
    DWORD cbSent=0;
    int i;
#endif    
    BYTE * pbIoBuffer = NULL;
    BYTE * pbFileBuffer = (BYTE *)pvBuffer;
    DWORD cbIoBufferLength = 0;
    DWORD cbIoBuffer;
    DWORD bytesRead=0,bytesSent=0;
    Message.ulVersion = SECBUFFER_VERSION;
    Message.cBuffers = 4;
    Message.pBuffers = Buffers;

    scRet = pDCtx->pDrv->hTLS->QueryAttributes(pDCtx->pDrv->hTLS,
				   pDCtx->hCtx, 
                                   SECPKG_ATTR_STREAM_SIZES,
                                   &Sizes);
    if(scRet != SEC_E_OK)
    {
        CPC_printf(&pDCtx->pDrv->Config,"**** Error 0x%x reading SECPKG_ATTR_STREAM_SIZES\n", scRet);
        return scRet;
    }

    CPC_printf(&pDCtx->pDrv->Config,"\nHeader: %d, Trailer: %d, MaxMessage: %d\n",
        Sizes.cbHeader,
        Sizes.cbTrailer,
        Sizes.cbMaximumMessage);

    //
    //   .  ,  EncryptMessage,
    //      'Sizes.cbMaximumMessage',  
    //     ,      
    //  .
    // 
    cbIoBufferLength = Sizes.cbHeader + 
                       Sizes.cbMaximumMessage +
                       Sizes.cbTrailer;//     
    //       cbIoBufferLength
    //    + 2048  . RFC 2246

    pbIoBuffer = CPCAlloc(&pDCtx->pDrv->Config, cbIoBufferLength, MP_WORK);
    if(pbIoBuffer == NULL)
    {
        CPC_printf(&pDCtx->pDrv->Config,"**** Out of memory (2)\n");
        return SEC_E_INTERNAL_ERROR;
    }


    cbIoBuffer = 0;
    for(bytesSent = 0;
	bytesSent < cbBuffer;
	bytesSent += bytesRead)
    {
	DWORD toSend=0;
	DWORD dwSent=0;
	bytesRead=My_min(Sizes.cbMaximumMessage,cbBuffer-bytesSent);

	memcpy(pbIoBuffer+Sizes.cbHeader,pbFileBuffer+bytesSent,bytesRead);

	Buffers[0].pvBuffer = pbIoBuffer;
	Buffers[0].cbBuffer = Sizes.cbHeader;
	Buffers[0].BufferType = SECBUFFER_STREAM_HEADER;

	Buffers[1].pvBuffer = pbIoBuffer + Sizes.cbHeader;
	Buffers[1].cbBuffer = bytesRead;
	Buffers[1].BufferType = SECBUFFER_DATA;

	Buffers[2].pvBuffer = pbIoBuffer + Sizes.cbHeader + bytesRead;
	Buffers[2].cbBuffer = Sizes.cbTrailer;
	Buffers[2].BufferType = SECBUFFER_STREAM_TRAILER;

	Buffers[3].BufferType = SECBUFFER_EMPTY;
        scRet = pDCtx->pDrv->hTLS->Seal(pDCtx->pDrv->hTLS,pDCtx->hCtx, 0, &Message, 0);

	if ( FAILED( scRet ) )
	  {
	    CPC_printf(&pDCtx->pDrv->Config,"kcreate_SA called\n"," EncryptMessage failed with 0x%x\n", scRet );
	    goto cleanup;
	  }
	  

	dwSent=0;
	toSend=Buffers[0].cbBuffer + Buffers[1].cbBuffer + Buffers[2].cbBuffer;
	do
	{
	    int err = emulate_send( Socket,
		    pbIoBuffer+dwSent,
		    toSend,
		    0 );
	    if (err < 0){
	      CPC_printf(&pDCtx->pDrv->Config, "send failed: %d\n", err );
	      goto cleanup;
	    } 
	    CPC_printf(&pDCtx->pDrv->Config,"\nSend %d data bytes to client\n", err);
	    dwSent+=(DWORD)err;
	    toSend-=(DWORD)err;
	}
	while(toSend>0);
//	PrintHexDump(16, IoBuffer);
    }
cleanup:    
    CPCFree(&pDCtx->pDrv->Config, pbIoBuffer, MP_WORK);
    return STATUS_SUCCESS;	
}    
