/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 111789 $
 * \date $Date: 2014-10-03 14:55:19 +0400 (Птн, 03 Окт 2014) $
 * \author $Author: sagafyin $
 * \brief   .
 */

#include "fkchdimgprj.h"

/*!
 * \ingroup fkchdimg_fun_reader
 * \brief   .
 * \param context [in]  . 
 * \param info [in/out]  #TReaderInfoFolderOpen
 * \sa TReaderInfoFolderOpen, READER_FUN_FOLDER_OPEN, 
 *  TSupSysFunction, fkchdimg_folder_close, #rdr_folder_open, #rdr_folder_close
 */
TSupErr fkchdimg_folder_open( 
    TSupSysContext *context, 
    TSupSysInfo *info )
{
    TReaderInfoFolderOpen *inf = (TReaderInfoFolderOpen*)info;
    TCHAR *folder;
    TCHAR *path;
    size_t folder_size;
    TFKCHDImgContext *ctx = (TFKCHDImgContext*)context;
    int exist;
    TSupErr err = 0;

    SUPSYS_PRE_CONTEXT( context, TFKCHDImgContext );
    SUPSYS_PRE_INFO( info, TReaderInfoFolderOpen );
    //SUPSYS_PRE( ctx->folder == NULL );

    //inf->size_of = sizeof( TReaderInfoFolderOpen );
    folder = inf->name.text;

    err = create_lock (folder, ctx);
    if (err != SUP_ERR_NO)
	return err;
    if( !folder )
	return SUP_ERR_NO;

    folder_size = inf->name.length;

    SUPSYS_PRE_READ_PTRS(folder, (folder_size + 1 ) * sizeof( TCHAR ));
    SUPSYS_PRE(folder_size == _tcslen(folder));

    path = (TCHAR*)malloc( ( 12 + folder_size + 1
			     + 1 + _tcslen( ctx->path ) + 1
			     + 1 ) * sizeof( TCHAR ) );
    if( path == NULL )
    {
	close_lock(ctx);
	return SUP_ERR_MEMORY;
    }
    ctx->folder = (TCHAR*)malloc( ( folder_size + 1 ) * sizeof( TCHAR ) );
    if( ctx->folder == NULL )
    {
	free( path ); path = NULL;
	close_lock(ctx);
	return SUP_ERR_MEMORY;
    }
    _tcscpy( path, ctx->path );
    _tcscat( path, folder );
    _tcscpy( ctx->folder, folder );

    /*   . */
    exist = fkchdimg_is_dir_exist( path, folder, ctx->cardctx->provname );
    if( !inf->mode->o_creat && ( exist == DIR_INVALID || !exist ) )
    {
	free( path );
	if (ctx->folder) {
	    free( ctx->folder ); ctx->folder = NULL;
	}
	close_lock(ctx);
	return RDR_ERR_FILE_NOT_FOUND;
    }
    if( inf->mode->o_creat && ( exist == DIR_INVALID || exist ) )
    {
	free( path );
	if (ctx->folder) {
	    free( ctx->folder ); ctx->folder = NULL;
	}
	close_lock(ctx);
	return RDR_ERR_FILE_EXIST;
    }
    fkchdimg_infofile_read(ctx);
    if( !inf->mode->o_creat )
    {
	free( path );
	return SUP_ERR_NO;
    }
#ifdef UNIX
    if (support_thread_actualize_uids()) {
	free( path );
	if (ctx->folder) {
	    free( ctx->folder ); ctx->folder = NULL;
	}
	close_lock(ctx);
	return SUP_ERR_RIGHTS;
    }
    if( _tmkdir( path, ((TFKCHDImgContext *)ctx)->machine?S_IRWXG|S_IRWXU:S_IRWXU ) )
#else
    if( !CreateDirectory( path, NULL ) )
#endif /* UNIX */
    {
	TSupErr code = fkchdimg_os_error();
	support_thread_deactualize_uids();
	free( path );
	if (ctx->folder) {
	    free( ctx->folder ); ctx->folder = NULL;
	}
	close_lock(ctx);
	return code;
    }
#ifdef UNIX
    if(chmod(path,((TFKCHDImgContext *)ctx)->machine?S_IRWXG|S_IRWXU:S_IRWXU)){
	TSupErr code = fkchdimg_os_error();
	support_thread_deactualize_uids();
	free( path );
	free( ctx->folder ); ctx->folder = NULL;
	close_lock(ctx);
	return code;
    }
    support_thread_deactualize_uids();
#endif /* UNIX */
    free( path );
    return SUP_ERR_NO;
}


static int is_registry_container_exist(const TCHAR *name, const TCHAR *provname)
{
    DWORD err = 0;
    int ret;
    HCRYPTPROV	hFKC=0;
    TCHAR* contname = NULL;
#if defined UNIX
    const TCHAR pref[] = _TEXT("\\\\.\\hdimage\\");
#else
    const TCHAR pref[] = _TEXT("\\\\.\\REGISTRY\\");
#endif /* UNIX */
    const size_t namelength  = sizeof(TCHAR)*(_tcslen(name)+_tcslen(pref)+1);

    contname = malloc (namelength);
    if (!contname) return SUP_ERR_MEMORY;

    _tcscpy(contname, pref);
    _tcscat(contname, name);

    ret = DIR_EXIST;
    if(!CryptAcquireContext(&hFKC, contname, provname, PROV_GOST_2001_DH, CRYPT_TOKEN_SHARED)) 
    {
	err = GetLastError();
	if (err == (DWORD)NTE_KEYSET_NOT_DEF)
	{
	    ret = DIR_NOTEXIST;
	} else 
	{
	    ret = DIR_INVALID;
	}
    } else
    {
	CryptReleaseContext(hFKC,0);
    }
    free (contname);
    return ret;
}

#ifdef UNIX
int fkchdimg_is_dir_exist( const TCHAR *path, const TCHAR * contname, const TCHAR * provname )
{ 
    struct stat buf;
    TSupErr code;
    
    code=support_thread_actualize_uids();
    if (code)
	return code;
	
    if (stat(path, &buf)) 
    {
	int err=errno;
	support_thread_deactualize_uids();
	return (err==ENOENT)?DIR_NOTEXIST:DIR_INVALID;
    }
    support_thread_deactualize_uids();
    if (!S_ISDIR(buf.st_mode))
    {
	return DIR_INVALID;
    }
    if (provname && contname)
	return is_registry_container_exist(contname, provname);
    else return DIR_EXIST;
}
#elif defined _WIN32
int fkchdimg_is_dir_exist( const TCHAR *path, const TCHAR * contname, const TCHAR * provname )
{
    DWORD attrib;
    attrib = GetFileAttributes( path ); 
    if( attrib == 0xffffffff ) 
	return DIR_NOTEXIST;
    if( !( attrib & FILE_ATTRIBUTE_DIRECTORY ) )
	return DIR_INVALID;
    if( attrib & ( FILE_ATTRIBUTE_TEMPORARY | FILE_ATTRIBUTE_SYSTEM
	| FILE_ATTRIBUTE_HIDDEN ) )
	return DIR_INVALID;
    if (provname && contname)
	return is_registry_container_exist(contname, provname);
    else return DIR_EXIST;
}
#else
#error fkchdimg_is_dir_exist not implemented.
#endif /* _WIN32, UNIX */
