/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 67945 $
 * \date $Date: 2011-05-10 11:40:17 +0400 (Втр, 10 Май 2011) $
 * \author $Author: dim $
 * \brief   -
 */

#include "fkchdimgprj.h"

#define FKC_INFO_FILE_NAME _TEXT("fkcinfo.inf")
#define FKC_HEADER_FILE_NAME _TEXT("header.key")
#define DWORDLENGTH 4

/* DWORD    */

static DWORD getDWORD (BYTE * src) {
    return ((DWORD)src[0])+(((DWORD)src[1])<<8)+(((DWORD)src[2])<<16)+(((DWORD)src[3])<<24);
}

static void setDWORD (BYTE * dst, DWORD src) {
    dst[0] = (BYTE)(src & 0xff);
    dst[1] = (BYTE)((src>>8) & 0xff);
    dst[2] = (BYTE)((src>>16) & 0xff);
    dst[3] = (BYTE)((src>>24) & 0xff);
}

/*         */

static size_t fromfile(BYTE * dst, size_t * dstlen, BYTE * src, size_t srclen) {
    size_t len;

    len = getDWORD (src);
    if (srclen < (len + DWORDLENGTH)) return len + DWORDLENGTH;
    src += DWORDLENGTH;
    memcpy(dst,src,len);
    *dstlen = len;
    return len + DWORDLENGTH;
}

static void tofile(BYTE * dst, size_t * dstlen, BYTE * src, size_t srclen) {
    setDWORD (dst, (DWORD)srclen);
    dst += DWORDLENGTH;
    if (srclen) memcpy(dst,src,srclen);
    *dstlen = srclen + DWORDLENGTH;
}

typedef enum fileflgs_ {
    FFLGRead,
    FFLGWrite
} fileflgs;

/*     */
static void makeFileParams (TReaderInfoOpen * setinfo, fileflgs flg) 
{
    memset( setinfo->mode, 0, sizeof( TReaderInfoOpenFlags ) );
    switch( flg )
    {
    case FFLGWrite:
	setinfo->mode->o_write = 1; 
	setinfo->mode->o_creat = 1;
	setinfo->mode->o_trunc = 1;
	break;
    case FFLGRead:
	setinfo->mode->o_read = 1; 
	break;
    }

    setinfo->size_of = sizeof( TReaderInfoOpen );
    setinfo->name.length = _tcslen( FKC_INFO_FILE_NAME );
    setinfo->name.text = FKC_INFO_FILE_NAME;

    setinfo->mode_bits = READER_MODEFLAG_BITS;
}

/*   */
static TSupErr saveARRAY (TFKCHDImgContext *ctx, BYTE * safe, size_t safelength)  
{
    size_t aslength;
    TReaderInfoWrite wrt;
    TSupErr code;
    TReaderInfoLength len;
    BYTE arrsafe [2048];

    len.size_of = sizeof(TReaderInfoLength);
    wrt.size_of = sizeof(TReaderInfoWrite);

    tofile(arrsafe, &aslength, safe, safelength);

    code = fkchdimg_length((TSupSysContext *)ctx, (TSupSysInfo*)&len);
    if (code) goto done;
    wrt.from = len.length;
    wrt.info.info = arrsafe;
    wrt.info.length = aslength;
    code = fkchdimg_write((TSupSysContext *)ctx, (TSupSysInfo*)&wrt);

done:
    return code;
}

/*   ,   */
static TSupErr loadARRAY (BYTE * desttemp, size_t *destlen, BYTE**pos, size_t * maxlen)
{
    size_t declen;

    declen = fromfile(desttemp, destlen, *pos, *maxlen);
    if (declen > *maxlen)
	return SUP_ERR_PARAM;
    else {
	*maxlen -= declen;
	*pos += declen;
    }
    return SUP_ERR_NO;
}

/* DWORD*/
static TSupErr saveDWORD (TFKCHDImgContext *ctx, DWORD safe) 
{
    BYTE arrdword[DWORDLENGTH];
    setDWORD(arrdword, safe); 
    return saveARRAY(ctx, arrdword, DWORDLENGTH);
}

/* DWORD*/
static TSupErr loadDWORD (BYTE**pos, size_t * maxlen, DWORD *load) 
{
    TSupErr code;
    BYTE arr[2048];
    size_t destlen;
    code = loadARRAY (arr, &destlen, pos, maxlen);
    if (code) return code;
    if (destlen != DWORDLENGTH) return SUP_ERR_PARAM;
    *load = getDWORD (arr);
    return SUP_ERR_NO;
}

/* NULL*/
static TSupErr saveNULL (TFKCHDImgContext *ctx) 
{
    return saveARRAY(ctx, NULL, 0);
}

/* TCHAR* */
static TSupErr savePTCHAR (TFKCHDImgContext *ctx, TCHAR * safe) 
{
    return safe ? saveARRAY(ctx, (BYTE*)safe, (_tcslen(safe)+1)*sizeof(TCHAR)) : saveNULL(ctx);
}

/* TCHAR* */
static TSupErr loadPTCHAR (BYTE**pos, size_t * maxlen, TCHAR * load, BOOL * isNull) 
{
    TSupErr code;
    BYTE arr[2048];
    size_t destlen = 0;
    code = loadARRAY (arr, &destlen, pos, maxlen);
    if (code) return code;
    if (destlen == 0) {
	*isNull = TRUE;
    } else {
	*isNull = FALSE;
	memcpy(load, arr, destlen);
    }
    return SUP_ERR_NO;
}

static TSupErr loadArrNull(BYTE**pos, size_t * maxlen, BYTE * load, BOOL * isNull) 
{
    TSupErr code;
    size_t destlen = 0;
    code = loadARRAY (load, &destlen, pos, maxlen);
    if (code) return code;
    if (destlen == 0) {
	*isNull = TRUE;
    } else {
	*isNull = FALSE;
    }
    return SUP_ERR_NO;
}

/* char* */
static TSupErr savePChar (TFKCHDImgContext *ctx, LPCSTR safe)
{
    return safe ? saveARRAY(ctx, (BYTE*)safe, (strlen(safe)+1)*sizeof(char)) : saveNULL(ctx);
}

/* char* */
static TSupErr loadPChar (BYTE**pos, size_t * maxlen, LPCSTR load, BOOL * isNull) 
{
    return loadPTCHAR(pos, maxlen, (TCHAR*)load, isNull);
}

/* BOOL*/
static TSupErr saveBOOL (TFKCHDImgContext *ctx, BOOL safe)
{
    return saveDWORD(ctx, safe?1:0);
}

/* BOOL*/
static TSupErr loadBOOL (BYTE**pos, size_t * maxlen, BOOL *load) 
{
    DWORD ret;
    TSupErr code;
    code = loadDWORD(pos,maxlen,&ret);
    if (code!=SUP_ERR_NO) return code;
    switch (ret) 
    {
    case 1: 
	*load = TRUE;
	break;
    case 0: 
	*load = FALSE;
	break;
    default:
	return SUP_ERR_PARAM;
    }
    return SUP_ERR_NO;
}

TSupErr init_read(TFKCHDImgContext *ctx )
{
    TReaderInfoFolderClose fcl;
    DWORD ret = SUP_ERR_NO;

    if (ctx->cont.opened == FALSE) {
	memset(&fcl, 0, sizeof(fcl));
	fcl.size_of = sizeof(TReaderInfoFolderClose);

	ret = fkchdimg_infofile_read(ctx);
	if (ret!=SUP_ERR_NO) {
	    goto done;
	}
	if (ctx->folder) {
	    ret = IntFKCTContainer(ctx, ctx->names.RealName, FALSE);
	    if (ret!=SUP_ERR_NO) {
		ret = SUP_ERR_CANCEL;
		goto done;
	    }
	}
	ctx->cont.opened = TRUE;
    }
done:
    return ret;
}

/*  */
TSupErr fkchdimg_infofile_read (TFKCHDImgContext *ctx )
{
    TSupErr code;
    TReaderInfoOpenFlags fflags;
    TReaderInfoOpen finfo;
    TReaderInfoLength length;
    TReaderInfoRead readinfo;
    TReaderInfoClose cls;
    BYTE * filearr = NULL;
    BYTE * pos;
    size_t fposit;
    size_t alllength;
    BOOL isNull;
    BYTE point[2048];

    finfo.size_of = sizeof(TReaderInfoOpen);
    length.size_of = sizeof(TReaderInfoLength);
    readinfo.size_of = sizeof(TReaderInfoRead);
    cls.size_of = sizeof(TReaderInfoClose);
    
    finfo.mode = &fflags;
    makeFileParams(&finfo,FFLGRead);

    code = fkchdimg_open((TSupSysContext *)ctx, (TSupSysInfo*)&finfo);
    if (code != SUP_ERR_NO) goto done;
    code = fkchdimg_length((TSupSysContext *)ctx, (TSupSysInfo*)&length);
    if (code != SUP_ERR_NO) goto done;
    filearr = (BYTE*)malloc(length.length);
    alllength = length.length;
    if (filearr==NULL) {
	code = SUP_ERR_MEMORY;
	goto done;
    }
    readinfo.from = 0;
    readinfo.info.info = filearr;
    readinfo.info.length = length.length;
    code = fkchdimg_read( (TSupSysContext *)ctx, (TSupSysInfo*)&readinfo);
    if (code != SUP_ERR_NO) goto done;
    fposit = length.length - readinfo.info.length;

    while (readinfo.info.length > 0) {
	readinfo.info.info = filearr + fposit;
	readinfo.from = fposit;
	code = fkchdimg_read( (TSupSysContext *)ctx, (TSupSysInfo*)&readinfo);
	if (code != SUP_ERR_NO) goto done;
	fposit = length.length - readinfo.info.length;
    }
    cls.size_of = sizeof(TReaderInfoClose);
    code = fkchdimg_close( (TSupSysContext *)ctx, (TSupSysInfo*)&cls);
    if (code != SUP_ERR_NO) goto done;
    pos = filearr;

    code = loadDWORD(&pos, &alllength, &ctx->cont.C.c_dh);
    if (code != SUP_ERR_NO) goto done;
    code = loadDWORD(&pos, &alllength, &ctx->cont.C.c_eke);
    if (code != SUP_ERR_NO) goto done;
    code = loadDWORD(&pos, &alllength, &ctx->cont.C.c_fls);
    if (code != SUP_ERR_NO) goto done;
    code = loadDWORD(&pos, &alllength, &ctx->cont.C.c_seqfls);
    if (code != SUP_ERR_NO) goto done;
    code = loadDWORD(&pos, &alllength, &ctx->cont.C.c_sig);
    if (code != SUP_ERR_NO) goto done;
    code = loadBOOL(&pos, &alllength, &ctx->cont.abilityInitKeys);
    if (code != SUP_ERR_NO) goto done;
    code = loadBOOL(&pos, &alllength, &ctx->cont.abilityWriteKeys);
    if (code != SUP_ERR_NO) goto done;
    code = loadBOOL(&pos, &alllength, &ctx->cont.disablePass);
    if (code != SUP_ERR_NO) goto done;

    code = loadPTCHAR(&pos, &alllength, ctx->cont.defaultPass, &isNull);
    if (code != SUP_ERR_NO) goto done;
    code = loadPChar(&pos, &alllength, (LPCSTR)ctx->cont.pszKeyOID,&isNull);
    if (code != SUP_ERR_NO) goto done;

    code = loadDWORD(&pos, &alllength, &ctx->cont.dwKeySpec);
    if (code != SUP_ERR_NO) goto done;

    ctx->names.FriendlyName = (TCHAR*)malloc((NAME_LENTH+1)*sizeof(TCHAR));
    code = loadPTCHAR(&pos, &alllength, ctx->names.FriendlyName, &isNull);
    if (code != SUP_ERR_NO) goto done;
    if (isNull) {
	free (ctx->names.FriendlyName);
	ctx->names.FriendlyName = NULL;
    }

    code = loadPTCHAR(&pos, &alllength, ctx->names.RealName, &isNull);
    if (code != SUP_ERR_NO) goto done;
    code = loadArrNull(&pos, &alllength, point, &isNull);
    if (code != SUP_ERR_NO) goto done;
    if (ctx->cont.pbPWPublicKeyBlob)
	free (ctx->cont.pbPWPublicKeyBlob);
    ctx->cont.pbPWPublicKeyBlob = NULL;
    if (!isNull) {
	ctx->cont.pbPWPublicKeyBlob = malloc(PUBLICBLOBLEN);
	memcpy(ctx->cont.pbPWPublicKeyBlob,point,PUBLICBLOBLEN);
    } 
    code = loadArrNull(&pos, &alllength, point, &isNull);
    if (code != SUP_ERR_NO) goto done;
    if (ctx->cont.pbSaltPublicBlob)
	free (ctx->cont.pbSaltPublicBlob);
    ctx->cont.pbSaltPublicBlob = NULL;
    if (!isNull) {
	ctx->cont.pbSaltPublicBlob = malloc(PUBLICBLOBLEN);
	memcpy(ctx->cont.pbSaltPublicBlob,point,PUBLICBLOBLEN);
    } 
    code = loadArrNull(&pos, &alllength, ctx->cont.syncro, &isNull);
    if (code != SUP_ERR_NO) goto done;
    if (isNull) {
	code = SUP_ERR_PARAM;
	goto done;
    }

done:
    if (filearr) free(filearr);
    return code;
}

TSupErr fkchdimg_infofile_clear (TFKCHDImgContext *ctx )
{
    TSupErr code;
    TReaderInfoUnlink inf;

    inf.size_of = sizeof(inf);
    inf.name.length = _tcslen( FKC_INFO_FILE_NAME );
    inf.name.text = FKC_INFO_FILE_NAME;
    code = fkchdimg_unlink(ctx,&inf);
    return code;
}

TSupErr fkchdimg_headerfile_clear (TFKCHDImgContext *ctx )
{
    TSupErr code;
    TReaderInfoUnlink inf;

    inf.size_of = sizeof(inf);
    inf.name.length = _tcslen( FKC_HEADER_FILE_NAME );
    inf.name.text = FKC_HEADER_FILE_NAME;
    code = fkchdimg_unlink(ctx,&inf);
    return code;
}

/*  */
TSupErr fkchdimg_infofile_write (TFKCHDImgContext *ctx )
{
    TSupErr code = SUP_ERR_NO;
    TReaderInfoOpenFlags fflags;
    TReaderInfoOpen finfo;
    TReaderInfoClose cls;

    memset(&fflags,0,sizeof(TReaderInfoOpenFlags));
    memset(&cls, 0, sizeof(TReaderInfoClose));
    cls.size_of = sizeof(TReaderInfoClose);

    memset(&finfo, 0, sizeof(TReaderInfoOpen));
    finfo.size_of = sizeof(TReaderInfoOpen);
    finfo.mode = &fflags;
    makeFileParams(&finfo, FFLGWrite);

    code = fkchdimg_open((TSupSysContext *)ctx, (TSupSysInfo*)&finfo);
    if (code != SUP_ERR_NO) goto done;

    code = saveDWORD(ctx, ctx->cont.C.c_dh);
    if (code != SUP_ERR_NO) goto done;
    code = saveDWORD(ctx, ctx->cont.C.c_eke);
    if (code != SUP_ERR_NO) goto done;
    code = saveDWORD(ctx, ctx->cont.C.c_fls);
    if (code != SUP_ERR_NO) goto done;
    code = saveDWORD(ctx, ctx->cont.C.c_seqfls);
    if (code != SUP_ERR_NO) goto done;
    code = saveDWORD(ctx, ctx->cont.C.c_sig);
    if (code != SUP_ERR_NO) goto done;
    code = saveBOOL(ctx, ctx->cont.abilityInitKeys);
    if (code != SUP_ERR_NO) goto done;
    code = saveBOOL(ctx, ctx->cont.abilityWriteKeys);
    if (code != SUP_ERR_NO) goto done;
    code = saveBOOL(ctx, ctx->cont.disablePass);
    if (code != SUP_ERR_NO) goto done;
    code = savePTCHAR(ctx, ctx->cont.defaultPass);
    if (code != SUP_ERR_NO) goto done;
    code = savePChar(ctx, (LPCSTR)ctx->cont.pszKeyOID);
    if (code != SUP_ERR_NO) goto done;
    code = saveDWORD(ctx, ctx->cont.dwKeySpec);
    if (code != SUP_ERR_NO) goto done;
    code = savePTCHAR(ctx, ctx->names.FriendlyName);
    if (code != SUP_ERR_NO) goto done;
    code = savePTCHAR(ctx, ctx->names.RealName);
    if (code != SUP_ERR_NO) goto done;

    if (ctx->cont.pbPWPublicKeyBlob)
	code = saveARRAY (ctx, ctx->cont.pbPWPublicKeyBlob, PUBLICBLOBLEN);
    else
	code = saveNULL (ctx);
    if (code != SUP_ERR_NO) goto done;

    if (ctx->cont.pbSaltPublicBlob)
	code = saveARRAY (ctx, ctx->cont.pbSaltPublicBlob, PUBLICBLOBLEN);
    else
	code = saveNULL (ctx);
    if (code != SUP_ERR_NO) goto done;

    code = saveARRAY (ctx, ctx->cont.syncro, SECRET_KEY_LEN);
    if (code != SUP_ERR_NO) goto done;

    /*TODO    */

    code = fkchdimg_close( (TSupSysContext *)ctx, (TSupSysInfo*)&cls);

done:
    return code;
}
