/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 76868 $
 * \date $Date: 2012-04-12 18:08:24 +0400 (Чтв, 12 Апр 2012) $
 * \author $Author: borodin $
 * \brief   
 */

#include "fkchdimgprj.h"


static DWORD make_fkchdimg_signature_maybedebug (TFKCHDImgContext *ctx, 
					   TReaderFkcSignInfo * si, 
					   const TRdrFkcOrder * k2,
					   BOOL isDebug) 
{
#if _DEBUG
    CRYPT_DATA_BLOB newblob;
    BYTE dataforeserved[SECRET_KEY_LEN];
#endif
    DWORD err = SUP_ERR_NO;

    if(isCNULL(ctx)) 
    {
	    err= SUP_ERR_CANCEL;
	    goto done;
    }

    if( !ctx->auth) {
	err = RDR_ERR_FKC_NEED_CONT_AUTH;
	goto done;
    }

    switch (si->id)
    {
    case TRECi_first:
	{
	    HCRYPTPROV hProvFKC = ctx->cont.hContainer;
	    DWORD dwHashLen = 32;
	    DWORD dwR1Len=sizeof(CRYPT_HASH_BLOB_EX);
	    BYTE pbHashVal[32];
	    CRYPT_HASH_BLOB_EX dbR2;
	    if (ctx->cont.signHash) return SUP_ERR_CANCEL;

	    SUPSYS_PRE_READ_PTRS( si->toFKC.first.e->v, dwHashLen );
	    SUPSYS_PRE_WRITE_PTRS( si->fromFKC.first.uR2->x, SECRET_KEY_LEN );
	    SUPSYS_PRE_WRITE_PTRS( si->fromFKC.first.uR2->y, SECRET_KEY_LEN );

	    if (si->toFKC.first.IsNewTr == FKC_TRANSACTION_IS_NEW) {
		ctx->cont.curmask = FKC_TRANS_SET_ZERO;
		ctx->cont.curtrid = si->toFKC.first.TrId;
	    } else {
		if (ctx->cont.curtrid) {
		    if (ctx->cont.curtrid != si->toFKC.first.TrId) {
			err = RDR_ERR_FKC_TRANSACTION;
			goto done;
		    }
		}
	    }

	    be2le(pbHashVal, (LPBYTE)si->toFKC.first.e->v, dwHashLen);

#if _DEBUG
	    if (isDebug) {
		be2le((LPBYTE)dataforeserved,(LPBYTE)k2->v,SECRET_KEY_LEN);
		newblob.cbData = SECRET_KEY_LEN;
		newblob.pbData = dataforeserved;
	    
		if(!CryptSetProvParam(hProvFKC, PP_RESERVED1,  (LPBYTE)&newblob, 0)) 
		{
		    err = GetLastError();
		    goto done;
		}
	    }
#else
	    UNUSED(isDebug);
	    UNUSED(k2);
#endif
	    if(!CryptCreateHash( hProvFKC, CALG_GR3411, 0, 0, &ctx->cont.signHash)){err = GetLastError();}
	    if(err) goto done;
	    if(!CryptSetHashParam( ctx->cont.signHash, HP_HASHVAL, pbHashVal, 0)){err = GetLastError();}
	    if(err) goto done;
	    if(!CryptSetHashParam( ctx->cont.signHash, HP_KEYSPEC_SIGN,(LPBYTE)&(ctx->cont.aAlgId),0)) {err = GetLastError();}
	    if(err) goto done;
	    if(!CryptGetHashParam( ctx->cont.signHash, HP_R2_SIGN,(LPBYTE)&dbR2,&dwR1Len,0)){err = GetLastError();}
	    if(err) goto done;
	        
	    be2le(si->fromFKC.first.uR2->x,dbR2.pbData,SECRET_KEY_LEN);
	    be2le(si->fromFKC.first.uR2->y,dbR2.pbData+SECRET_KEY_LEN,SECRET_KEY_LEN);
	}
	err = RDR_ERR_BLOCK;
	break;
    case TRECi_first_hash:
	err = SUP_ERR_UNSUPPORTED;
	break;
    case TRECi_second:
	{
	    DWORD dwSigLen=32;
	    BYTE pbSignature2[(2*SECRET_KEY_LEN)];
	    CRYPT_HASH_BLOB_EX dbR;

	    SUPSYS_PRE_READ_PTRS( si->toFKC.second.ur, SECRET_KEY_LEN );
	    SUPSYS_PRE_WRITE_PTRS( si->fromFKC.second.us2, SECRET_KEY_LEN );

	    if (ctx->cont.curtrid) {
		if (ctx->cont.curtrid != si->toFKC.second.TrId) {
		    err = RDR_ERR_FKC_TRANSACTION;
		    goto done;
		}
	    }

	    if (!ctx->cont.signHash) return SUP_ERR_CANCEL;
	    if(!ctx->cont.C.c_dh || !ctx->cont.C.c_eke || !ctx->cont.C.c_fls || !ctx->cont.C.c_seqfls || !ctx->cont.C.c_sig) {
		    err= SUP_ERR_CANCEL;
		    goto done;
	    }
	    /*be2le() -----*/
	    be2le(dbR.pbData,(LPBYTE)si->toFKC.second.ur->v,SECRET_KEY_LEN);
	    memset(dbR.pbData+SECRET_KEY_LEN,0,SECRET_KEY_LEN);
	    dbR.tPublicKeyParam.BlobHeader.aiKeyAlg = ctx->cont.aAlgId;
	    dbR.tPublicKeyParam.KeyParam.Magic = GR3410_1_MAGIC;
	    dbR.tPublicKeyParam.KeyParam.BitLen = sizeof(CRYPT_HASH_BLOB_EX)*8;
	    dbR.tPublicKeyParam.BlobHeader.bType = HASHPUBLICKEYEXBLOB;
	    dbR.tPublicKeyParam.BlobHeader.bVersion = BLOB_VERSION;

	    /* dbR <- dFKC_Data */
	    if(!CryptSetHashParam( ctx->cont.signHash, HP_R_SIGN,(LPBYTE)&dbR,0)) {
		err = GetLastError();
		goto done;
	    }
	    dwSigLen = (2*SECRET_KEY_LEN); //?
	    if(!CryptSignHash( ctx->cont.signHash, ctx->cont.dwKeySpec, NULL, 0, pbSignature2, &dwSigLen)) {
		err = GetLastError();
		goto done;
	    }
	    /*be2le() -----*/
	    be2le(si->fromFKC.second.us2->v,pbSignature2+SECRET_KEY_LEN,SECRET_KEY_LEN);

	    if(!CryptDestroyHash( ctx->cont.signHash)) {
		err = GetLastError();
		goto done;
	    }
	    ctx->cont.signHash = 0;
	    ctx->cont.C.c_sig--;
	}
	err = SUP_ERR_NO;
	break;
    }

done:
    if (err!= SUP_ERR_NO && err != RDR_ERR_BLOCK) {
	fkchdimg_infofile_write(ctx);
    }
    return err;

}

#if _DEBUG
TSupErr fkchdimg_signature_debug (
    TSupSysContext *context, TSupSysInfo *info )
{
    TReaderFkcSignInfo valsi;
    DWORD err = SUP_ERR_NO;
    TReaderFkcSignInfo_Debug * si_D = (TReaderFkcSignInfo_Debug *)info;
    TFKCHDImgContext *ctx = (TFKCHDImgContext*)context;
    const TRdrFkcOrder * k2 = NULL;
    BOOL isDbg = FALSE;

    SUPSYS_PRE_CONTEXT( context, TFKCHDImgContext );
    SUPSYS_PRE_INFO( info, TReaderFkcSignInfo_Debug );
    k2 = si_D->toFKC.first.k2;

    if (si_D->id == TRECi_first) {
	if (k2) {
	    isDbg = TRUE;
	}
	valsi.toFKC.first.e = si_D->toFKC.first.e;
	valsi.fromFKC.first.uR2 = si_D->fromFKC.first.uR2;
    } else {
	valsi.toFKC.second.ur = si_D->toFKC.second.ur;
	valsi.fromFKC.second.us2 = si_D->fromFKC.second.us2;
    }

    err = make_fkchdimg_signature_maybedebug (ctx, &valsi, k2, isDbg);

    return err;
}
#endif

TSupErr fkchdimg_signature ( 
    TSupSysContext *context, 
    TSupSysInfo *info )
{
    DWORD err = SUP_ERR_NO;
    TReaderFkcSignInfo * si = (TReaderFkcSignInfo *)info;
    TFKCHDImgContext *ctx = (TFKCHDImgContext*)context;

    SUPSYS_PRE_CONTEXT( context, TFKCHDImgContext );
    SUPSYS_PRE_INFO( info, TReaderFkcSignInfo );

    err = init_read(ctx);
    if (err) goto done;

    err = make_fkchdimg_signature_maybedebug (ctx, si, NULL, FALSE);

done:
    return err;
}


