/*
 * Copyright(C) 2000  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

#include "tst12prj.h" /*+ Project (READER/FAT12/TEST) include.
     include    (READER/FAT12/TEST). +*/
TSupportDbContext *support_context_io=NULL;

/*   . */
static TSupErr register_all( void );

/*   . */
static void unregister_all( void );

/*  . */
static TSupErr test_all( const TCHAR *reader, TCHAR *arg );

/*   . */
static TSupSysEList *reader_list = NULL;

static int std_main( const TCHAR *reader, TCHAR *arg );

/*++++
 *     .
 ++++*/
static TSupErr reader_test_info_system_flag_fkc(
    const TSupSysEContext *context ) /*+ (i) HANDLE . +*/
{
    TSupErr code; /*  . */
    int flag; /*  . */
    
    /*+     . +*/
    DbTrace(DB_TRACE, (FTEXT(support_context_io, "Getting reader flags...")));
    
    code = reader_test_info_system_flag(context);
    if( code )
	return test_error( context, code );

    /*+     . +*/
    code = supsys_flag( context, RDR_FLAG_BLOCK_RDR, RDR_FLAG_RDR_FKC_SUPPORTED,
	&flag );

    if( code )
	return test_error( context, code );
    if( flag )
    {
	DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC only.")));


	code = supsys_flag( context, RDR_FLAG_BLOCK_FKC, RDR_FLAG_FKC_NULL_FOLDER,
	    &flag );
	if( code )
	    return test_error( context, code );
	if( flag )
	{
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC support NULL folder.")));
	} else {
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC is not support NULL folder.")));
	}

	code = supsys_flag( context, RDR_FLAG_BLOCK_FKC, RDR_FLAG_FKC_DISABLE_PASSWORD,
	    &flag );
	if( code )
	    return test_error( context, code );
	if( flag )
	{
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC: password is disable.")));
	} else {
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC: password is enable.")));
	}

	code = supsys_flag( context, RDR_FLAG_BLOCK_FKC, RDR_FLAG_FKC_ENABLE_INIT_KEYS,
	    &flag );
	if( code )
	    return test_error( context, code );
	if( flag )
	{
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC is able to generate keys.")));
	} else {
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC is not able to generate keys.")));
	}

	code = supsys_flag( context, RDR_FLAG_BLOCK_FKC, RDR_FLAG_FKC_ENABLE_SET_KEYS,
	    &flag );
	if( code )
	    return test_error( context, code );
	if( flag )
	{
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC is able to set user's keys.")));
	} else {
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "FKC is not able to set user's keys.")));
	}

    }else{
	DbTrace(DB_TRACE, (FTEXT(support_context_io, "Reader is not support FKC.")));
    }   

    return SUP_ERR_NO;
}

int _tmain(int argc, TCHAR ** argv)
{
    int ret;
    support_context_io = support_print_init( NULL, DTEXT( "FKCHDIMG_TST" ),
	SUPPORT_DBLEVEL_ALL, SUPPORT_DBFMT_CLEAN );
    if (argc>1) {
	ret = std_main( argv[1], NULL);
    } else  {
	ret = std_main( _TEXT("FKCHDIMG"), NULL);
    }

    support_print_done( support_context_io );

    return ret;
}

static int std_main( const TCHAR *reader, TCHAR *arg )
{
    TSupErr code;

    code = support_load_library();
    if( code ) return code;
    code = rdr_load_library();
    if( code ) return code;
    code = register_all();
    if( code ) return code;
    code = test_all( reader, arg );

    unregister_all();
    rdr_unload_library();
    support_unload_library();

    return code;
}

/*!
 * \brief      .
 */
static TSupErr register_all( void )
{
    TSupErr code;

    code = rdr_register_start( &reader_list );
    if( code )
    {
	supsys_unregister_all( reader_list, 1 );
	return code;
    }
    code = supsys_register_all( reader_list );
    if( code )
    {
	supsys_unregister_all( reader_list, 1 );
	return test_error( NULL, code );
    }
    return SUP_ERR_NO;
}

/*!
 *      .
 */
static void unregister_all( void )
{
    supsys_unregister_all( reader_list, 1 );
}

/*!
 *  .
 * \return  .
 */
static TSupErr test_all( const TCHAR *reader, TCHAR *arg )
{
    TSupErr code;
    TSupSysEContext *context=NULL;
    TSupSysEContext *dupctx=NULL;
    LPCSTR pPW = "FKCPassWordSign";
    BOOL abletosetkey;
    BYTE testar[128];
    int i;
    BOOL lock = FALSE;

    code = supsys_find_nickname( reader_list, (TCHAR*)reader, arg, &context );
    if( code ) return test_error( NULL, code );
    code = supsys_test_info( context ); if( code ) goto done;
    code = reader_test_info( context ); if( code ) goto done;
    code = supsys_context_dup (context, &dupctx); if( code ) goto done;

    code = supsys_test_connect( context ); if( code ) goto done;
    code = reader_test_local_machine( context, 0 ); if( code ) goto done;
    code = reader_test_connect_carrier( context ); if( code ) goto done;

    code = supsys_test_connect( dupctx ); if( code ) goto done;
    code = reader_test_connect_carrier( dupctx ); if( code ) goto exit;

//#ifndef UNIX
//    code = reader_test_retry_wait_function_set( context ); if( code ) goto done;
//#endif

    code = reader_test_info_system_flag_fkc(context); if( code ) goto exit;
    code = reader_test_fkc_get_param(context,&abletosetkey); if( code ) goto exit;

    for (i = 0; i<128; i++) {
	testar[i] = (BYTE)(i+1);
    }

    code = reader_test_lock( context ); if( code ) goto exit;
    lock = TRUE;
    code = loginToNull(context, testar,1); if( code ) goto exit;
    code = reader_test_unlock( context ); 
    lock = FALSE;
    if( code ) goto exit;

    code = reader_test_lock( dupctx ); if( code ) goto exit;
    lock = TRUE;
    code = loginToNull(dupctx,testar,0); if( code ) goto exit;
    code = reader_test_unlock( dupctx );     
    lock = FALSE;
    if( code ) goto exit;


    code = reader_test_lock( context ); if( code ) goto exit;
    lock = TRUE;
    code = reader_test_folder_enum( context ); if( code ) goto exit;

    code = reader_test_folder_open( context, FKC_CONTAINER_NAME );
    if( !code )
    {
	code = reader_test_folder_clear( context ); if( code ) goto exit;
	DbTrace(DB_TRACE, (FTEXT(support_context_io, "Folder: %s deleted"),FKC_CONTAINER_NAME));
    }
    code = reader_test_folder_open( context, FKC_DUP_CONTAINER_NAME );
    if( !code )
    {
	code = reader_test_folder_clear( context ); if( code ) goto exit;
	DbTrace(DB_TRACE, (FTEXT(support_context_io, "Folder: %s deleted"),FKC_DUP_CONTAINER_NAME));
    }

    if (abletosetkey) {
	code = reader_test_folder_open( context, FKC_CONTAINER_NAME_T );
	if( !code )
	{
	    code = reader_test_folder_clear( context ); if( code ) goto exit;
	    DbTrace(DB_TRACE, (FTEXT(support_context_io, "Folder: %s deleted"),FKC_CONTAINER_NAME_T));
	}
    }

    code = Test_FKCAdminCreate(context, dupctx, (TCHAR*)FKC_CONTAINER_NAME, AT_SIGNATURE, 
	(char*)szOID_GostR3410_2001_CryptoPro_A_ParamSet, (LPSTR)pPW, FALSE);
    if( code ) goto exit;
    printf ("First Full Test finished successfully\n");
    code = Test_FKCAdminCreate(context, dupctx, (TCHAR*)FKC_CONTAINER_NAME, AT_KEYEXCHANGE, 
	(char*)szOID_GostR3410_2001_CryptoPro_XchA_ParamSet, (LPSTR)pPW, FALSE);
    if( code ) goto exit;
    printf ("Second Full Test finished successfully\n");
    if (abletosetkey) {
	//setkey test;
	code = Test_FKCAdminCreate(context, dupctx, (TCHAR*)FKC_CONTAINER_NAME_T, AT_KEYEXCHANGE, 
	    (char*)szOID_GostR3410_2001_CryptoPro_XchA_ParamSet, (LPSTR)pPW, TRUE);
	printf ("Set Key Full Test finished successfully\n");
    }
    if( code ) goto exit;

    code = reader_test_unlock( context ); 
    lock = FALSE;
    if( code ) goto exit;

exit:
    if (lock) reader_test_unlock( context ); 
    code = reader_test_disconnect_carrier( dupctx ); if( code ) goto exit;
    code = supsys_test_disconnect( dupctx ); if( code ) goto exit;
    code = reader_test_disconnect_carrier( context ); if( code ) goto exit;
    code = supsys_test_disconnect( context ); if( code ) goto exit;
done:
    if (code) {
	printf ("\nTest failed!\n");
    } else {
	printf ("\nTest PASS!\n");
    }
    supsys_context_free( dupctx );
    supsys_context_free( context );

#if 0
#if defined UNIX
// destroy CryptoServiceProvider for valgrind' checks
    HCRYPTPROV uProv;
    if(!CryptAcquireContext(&uProv, _TEXT("$reboot$"), NULL, PROV_GOST_2001_DH, CRYPT_REBOOT|CRYPT_DELETEKEYSET)) {
	printf("\"$reboot$\", CRYPT_REBOOT fail [Err: 0x%08x]\n", GetLastError());
    }
#endif	// UNIX
#endif /* 0 */

    return code;
}
